# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
#
# This file is part of Satega (www.satega.org).
#
# aatdark <aatdark@gmail.com>
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
# Autor of this file:
# -> aatdark
#

"""Pylons middleware initialization"""
from beaker.middleware import CacheMiddleware, SessionMiddleware
from paste.cascade import Cascade
from paste.registry import RegistryManager
from paste.urlparser import StaticURLParser
from paste.deploy.converters import asbool
from pylons import config
from pylons.middleware import ErrorHandler, StatusCodeRedirect
from pylons.wsgiapp import PylonsApp
from routes.middleware import RoutesMiddleware

from satega.config.environment import load_environment

import logging
log = logging.getLogger(__name__)
log.setLevel(logging.WARNING); # set log level (TODO maybe create config variable)
#very BAD code here: should be removed soon (aatdark)
from satega.model.sategacd import SategaCd
x = SategaCd()
from satega.model.service import *
#BAD codes ends

def add_auth(app):
    """
    Add authentication and authorization middleware to the ``app``.

    :param app: The WSGI application.
    :return: The same WSGI application, with authentication and
        authorization middleware.

    People will login using HTTP Authentication and their credentials are
    kept in an ``Htpasswd`` file. For authorization through repoze.what,
    we load our groups stored in an ``Htgroups`` file and our permissions
    stored in an ``.ini`` file.

    """
    from satega.model.drive import DriveTools
    from repoze.who.plugins.basicauth import BasicAuthPlugin
    from repoze.who.plugins.friendlyform import FriendlyFormPlugin  #RedirectingFormPlugin
    from repoze.who.plugins.ldap import LDAPAttributesPlugin,LDAPAuthenticatorPlugin
    from repoze.who.plugins.auth_tkt import AuthTktCookiePlugin
    from repoze.what.plugins.groupldap import LDAPGroupSourceAdapter
    from repoze.what.middleware import setup_auth

    groups = {'all_groups': LDAPGroupSourceAdapter(config['ldap_url'],'ou=Users,dc=satega,dc=local','ou=Groups,dc=satega,dc=local')}
    permissions = {}

    # repoze.who identifiers; you may add as much as you need:
    #							    (login_form_url, login_handler_path, post_login_url, logout_handler_path, post_logout_url, rememberer_name, login_counter_name=None)
    formauth = FriendlyFormPlugin('/account/login','/account/dologin','/service/index','/account/logout','/account/shlogout','auth_tkt',login_counter_name='counter')
    auth_tkt = AuthTktCookiePlugin('extremsecret')
    identifiers = [('form_auth',formauth) ,  ('auth_tkt',auth_tkt)]

    challengers = [('form_auth', formauth)]
    ldap_auth = LDAPAuthenticatorPlugin(config['ldap_url'],
                                    'ou=Users,dc=satega,dc=local')
    authenticators = [('ldap_auth', ldap_auth)]


    ldap_attributes = LDAPAttributesPlugin(config['ldap_url'])
    mdproviders = [('ldap_attributes', ldap_attributes)]
    
    app_with_auth = setup_auth(
        app,
        groups,
        permissions,
        identifiers=identifiers,
        authenticators=authenticators,
        challengers=challengers,
        mdproviders = mdproviders,
        log_stream=log,
        log_level=logging.WARNING
    )
    
    #automount all partitions
    DriveTools.autoMountAllParts()
    
    return app_with_auth



def make_app(global_conf, full_stack=True, static_files=True, **app_conf):
    """Create a Pylons WSGI application and return it

    ``global_conf``
        The inherited configuration for this application. Normally from
        the [DEFAULT] section of the Paste ini file.

    ``full_stack``
        Whether this application provides a full WSGI stack (by default,
        meaning it handles its own exceptions and errors). Disable
        full_stack when this application is "managed" by another WSGI
        middleware.

    ``static_files``
        Whether this application serves its own static files; disable
        when another web server is responsible for serving them.

    ``app_conf``
        The application's local configuration. Normally specified in
        the [app:<name>] section of the Paste ini file (where <name>
        defaults to main).

    """
    # Configure the Pylons environment
    load_environment(global_conf, app_conf)

    #fix the strict tmp bug (satega)
    config['pylons.strict_tmpl_context'] = False
    config['pylons.strict_c'] = False
    # The Pylons WSGI app
    app = PylonsApp()

    # Routing/Session/Cache Middleware
    app = RoutesMiddleware(app, config['routes.map'])
    app = SessionMiddleware(app, config)
    app = CacheMiddleware(app, config)

    # CUSTOM MIDDLEWARE HERE (filtered by error handling middlewares)
    app = add_auth(app)
    # load the serviceList parts
    
    if asbool(full_stack):
        # Handle Python exceptions
        app = ErrorHandler(app, global_conf, **config['pylons.errorware'])

        # Display error documents for 401, 403, 404 status codes (and
        # 500 when debug is disabled)
        if asbool(config['debug']):
            app = StatusCodeRedirect(app)
        else:
            app = StatusCodeRedirect(app, [400, 401, 403, 404, 500])

    # Establish the Registry for this application
    app = RegistryManager(app)

    if asbool(static_files):
        # Serve static files
        static_app = StaticURLParser(config['pylons.paths']['static_files'])
        app = Cascade([static_app, app])

    return app
