# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
#
# This file is part of Satega. 
#
# aatdark <aatdark@gmail.com>
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
# Autor of this file:
# -> aatdark
#

import logging

from pylons import request, response, session, tmpl_context as c
from pylons.controllers.util import abort, redirect_to
from repoze.what.predicates import has_permission, in_group
from repoze.what.plugins.pylonshq import ActionProtector,ControllerProtector
import satega.lib.helpers as h
from satega.lib.base import BaseController, render

from satega.model.user import *

import formencode
from pylons.decorators import validate
log = logging.getLogger(__name__)

import Image
class SategaUser(formencode.Schema):
    allow_extra_fields = True
    filter_extra_fields = True
    if_key_missing=False

    pw = formencode.validators.MinLength(3)
    pw2 = formencode.validators.MinLength(3)
    #pw2 = formencode.validators.FieldsMatch('pw','pw2')  #,not_empty=True
    chained_validators = [formencode.validators.FieldsMatch('pw', 'pw2')]
    admin = formencode.validators.StringBool(if_empty=False,not_empty=False)
    pic = formencode.validators.FieldStorageUploadConverter(not_empty=False)

class ValidCreateUser(SategaUser):
    uid = formencode.All (
            formencode.validators.PlainText(not_empty=True),
            UniqueUsername())

class ValidUpdateUser(SategaUser):
    pass

class AccountController(BaseController):

    def login(self):
        identity = request.environ.get('repoze.who.identity')
        came_from = request.params.get('came_from', None)
        if identity is not None:
            if came_from:
                redirect_to(str(came_from))
            redirect_to(h.url_for(controller='service',action='index'))
        login_counter = request.environ.get('repoze.who.logins')
        if login_counter > 0:
            if LdapControl.ping() == False:
                c.error = 'The authentification server (ldap) can not be reached'
            else:
                c.error = 'The username and/or the password did not match'
        if came_from and came_from != "/":
            c.message = 'You have to login to view this resource'
        return render('/login.mako')

    def shlogout(self):
            c.message = 'You have been logged out'
            return render('/login.mako')

    @ActionProtector(in_group('admins'))
    def index(self):
        c.users = LdapControl.listUsers()
        return render('/userlist.mako')

    @ActionProtector(in_group('admins'))
    def showimage(self,id):
        uid = id
        jpg = None
        try:
            jpg = LdapControl.getUserImage(LdapControl.makeDN(uid))
        except :
            pass
        response.headers['Content-Type'] = 'image/png'
        if jpg == None:
            return redirect_to("/start-here.png")
        return jpg

    @ActionProtector(in_group('admins'))
    def deleteimg(self,id):
        """
        removes the image in the ldap server
        """
        LdapControl.setUserImage(LdapControl.makeDN(id),'')

        session['flash'] = 'Removed Img'
        session.save()
        return self._success('account' , 'index' , 'removed image')

    @ActionProtector(in_group('admins'))
    def delete(self,id):
        """
        remove the in id
        """
        LdapControl.deleteUser(id)
        return self._success('account' , 'index' , 'Deleted User %s' % (id))

    """Change a user"""
    @ActionProtector(in_group('admins'))
    def change(self,id):
        """
        show the change user form
        """
        user = LdapControl.loadUser(id)
        if user == None:
            return self._error('/userlist.mako' , 'The User was not found in the System (%s)' % (id))

        admin = int(user.isAdmin)
        defaults = {'uid':user.uid(),'admin':admin}
        c.createuser = False
        c.id = id
        return formencode.htmlfill.render(render('/useredit.mako'),defaults)

    @ActionProtector(in_group('admins'))
    @validate(schema=ValidUpdateUser(),form='change',post_only=False,on_get=True)
    def submitchange(self,id):
        """
        id => user to change
        pw: if empty don't change
        admin: new admin flag
        """
        ldapConnection = LdapControl.getLdapConnection('root','satega')
        pw = self.form_result['pw']
        admin = self.form_result['admin']
        #root is always admin

        if id == 'root':
            admin = True
        user = LdapControl.loadUser(id)
        if pw != None:
            user.setPassword(pw)

        pic = self.form_result['pic']
        if pic != None:
            #this code ensures that the uploaded file is an image
            im = Image.open(pic.file)
            #print im.format, im.size, im.mode
            pic.file.seek(0)
            pic_data = pic.file.read()
            user.setImg(pic_data)

        user.setAdmin(admin)
        user.save(ldapConnection)
        ldapConnection.unbind_s()
        return self._success('account' , 'index' , 'updated %s'  % id)

    """Create a user"""
    @ActionProtector(in_group('admins'))
    @validate(schema=ValidCreateUser(),form='create',post_only=False,on_get=True)
    def submitcreate(self):
        ldapConnection = LdapControl.getLdapConnection('root','satega')
        pw = self.form_result['pw']
        uid = self.form_result['uid']
        admin = self.form_result['admin']

        #root is always admin
        attributes = {'dn':LdapControl.makeDN(uid) , 'uid':(uid.encode(),)}
        user = LdapUser( attributes, admin)
        user.setPassword(pw)
        pic = self.form_result['pic']
        if pic != None:
            #this code ensures that the uploaded file is an image
            im = Image.open(pic.file)
            #print im.format, im.size, im.mode
            pic.file.seek(0)
            pic_data = pic.file.read()
            user.setImg(pic_data)

        user.save(ldapConnection)
        ldapConnection.unbind_s()
        return self._success('account' , 'index' , 'User %s created' % (uid))

    @ActionProtector(in_group('admins'))

    def create(self):
        """
        shows create new user form
        """
        c.createuser = True
        return render('/useredit.mako')

