# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
#
# This file is part of Satega (www.satega.org)
#
# aatdark <aatdark@gmail.com>
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
# Autor of this file:
# -> aatdark
#

#from ldap.schema.subentry import ldap_entry
from satega.model.boxbackup import Boxbackup , AccountNameMapping
from satega.model.background import TaskControl
from pylons import request, response, session, tmpl_context as c
from pylons import config
from pylons.controllers.util import abort, redirect_to
from repoze.what.predicates import has_permission, in_group
from repoze.what.plugins.pylonshq import ActionProtector,ControllerProtector
import satega.lib.helpers as h
from satega.lib.base import BaseController, render , XMLRPCController

import ldap

import formencode
from pylons.decorators import validate
import base64
import logging
log = logging.getLogger(__name__)

class BackupxmlController(XMLRPCController):
    """
    xml rpc interface for the backup module
    auth: every call must post the user, password
    """
    def get_account_list(self,dn,password):
        """
        @param dn the username of the satega admin user
        @param password: the password of the user
        returns the ids and the accout names
        @returns an dict with key: id , value : account name
        """
        if self._check_auth(dn,password) == False:
            return 'not authed'
        print dn,password
        ret = {}
        for id in Boxbackup.get_account_list():
            ret[str(id)] = AccountNameMapping.get_name_from_id(id) or ''
        return ret

    def get_client_certs(self,dn,password,id):
        """
        @param id : the hex backup id
        return the ca cert
        """
        if self._check_auth(dn,password) == False:
            return 'not authed'

        certarray =  Boxbackup.get_certs_for_client(id)
        ret = {}
        ret['raw'] = base64.b64encode(certarray['raw']) 
        ret['key'] = base64.b64encode(certarray['key'])
        ret['cert'] = base64.b64encode(certarray['cert'])
        ret['ca'] = base64.b64encode(certarray['ca'])
        return ret

    def _check_auth(self,dn,password):
        ldap_connection = ldap.initialize(config['ldap_url'])
        try:
            ldap_connection.simple_bind_s(dn, password)
        except ldap.LDAPError:
            return False
        return True
    
    @ActionProtector(in_group('admins'))
    def index(self):
    	return Boxbackup.get_certs_for_client(1)['cert']




