# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
#
# This file is part of Satega.
#
# aatdark <aatdark@gmail.com>
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
# Autor of this file:
# -> aatdark
#


from satega.model.boxbackup import Boxbackup , AccountNameMapping
from satega.model.background import TaskControl
from pylons import request, response, session, tmpl_context as c
from pylons.controllers.util import abort, redirect_to
from repoze.what.predicates import has_permission, in_group
from repoze.what.plugins.pylonshq import ActionProtector,ControllerProtector
import satega.lib.helpers as h
from satega.lib.base import BaseController, render

from satega.model.user import *

import formencode
from pylons.decorators import validate
from satega.lib.validatorhelper import GreaterThan

import logging
log = logging.getLogger(__name__)



        
class BackupSchema(formencode.Schema):
    allow_extra_fields = True
    filter_extra_fields = True
    if_key_missing = False  
    name = formencode.validators.String(not_empty = True)
    soft = formencode.validators.Number(not_empty = True)
    hard = formencode.validators.Number(not_empty = True)
    chained_validators = [ GreaterThan('hard','soft')]

class BackupSchemaUpdate(BackupSchema):
    id = formencode.validators.Int()

class BackupSchemaCreate(BackupSchema):
    id = formencode.validators.Int(if_empty= None ,not_empty = False)
    
class BoxbackupController(BaseController):

    @ActionProtector(in_group('admins'))
    def index(self):
        c.accounts = Boxbackup.get_account_objects()
        return render('/boxbackup.mako')



    @ActionProtector(in_group('admins'))
    def change(self,id):
        c.update = True
        c.update_id = id
        account = Boxbackup.get_account_object(id)
        c.name = account.name
        defaults = { 'name' : account.name ,
                     'id' : id,
                     'soft' : account.soft(),
                     'hard' : account.hard(),
                    }
        return formencode.htmlfill.render(render('/boxbackupcreate.mako'),defaults)

    @ActionProtector(in_group('admins'))
    @validate(schema=BackupSchemaUpdate(),form='change',post_only=False,on_get=True)
    def submitchange(self,id):
        computer_name = self.form_result['name']
        soft_limit = self.form_result['soft']
        hard_limit = self.form_result['hard']
        Boxbackup.update_account(id, computer_name, soft_limit, hard_limit)
        return self._success('boxbackup', 'index', 'Successfully updated share %s (%s)' % (computer_name , id))





    @ActionProtector(in_group('admins'))
    def create(self):
        """
        shows an form for the user to create an new account
        """
        c.update = False
        return render('/boxbackupcreate.mako')

    @ActionProtector(in_group('admins'))
    @validate(schema=BackupSchemaCreate(),form='create',post_only=False,on_get=True)
    def submitcreate(self):
        id = self.form_result['id']
        name = self.form_result['name']
        soft_limit = self.form_result['soft']
        hard_limit = self.form_result['hard']
        id = Boxbackup.create_account_and_certs(name, soft_limit, hard_limit, id)
        return self._success('boxbackup', 'index', 'Created Account %d' % (id))
    
    @ActionProtector(in_group('admins'))
    def certs(self,id,typ):
        """
        shows an page where the user can download
        the certs for this account
        """
        c.id = id;
        c.name = AccountNameMapping.get_name_from_id(id)
        if typ == None:
            # show the landing page
            c.ca = Boxbackup.get_filename_for_cert(id, 'ca')
            c.key = Boxbackup.get_filename_for_cert(id, 'key')
            c.cert = Boxbackup.get_filename_for_cert(id, 'cert')
            c.raw = Boxbackup.get_filename_for_cert(id, 'raw')
            return render('/boxbackup_certs.mako')
        if typ not in ('ca', 'raw' , 'cert' , 'key'):
            return self._error('boxbackup', 'certs', 'Typ of request not recognized')
        content = Boxbackup.get_certs_for_client(int(id))[typ]
        filename = Boxbackup.get_filename_for_cert(id, typ)
        response.headers['Content-Type'] = 'application/force-download'
        response.headers['Content-Disposition'] = 'attachment;filename=%s'% (filename,)
        return content

    @ActionProtector(in_group('admins'))
    def delete(self,id):
        """
        shown an are you sure dialog
        with some data about the account
        to be deleted
        """
        c.id = id
        c.name = AccountNameMapping.get_name_from_id(id)
        print c.name
        return render('/boxbackupdelete.mako')

    @ActionProtector(in_group('admins'))
    def submitdelete(self,id):
        """
        delelete the backup
        """
        if 'delete' in request.params:
            name = AccountNameMapping.get_name_from_id(id) or 'noname'
            Boxbackup.delete_account(id)
            return self._success('boxbackup', 'index', 'removed %s (%s)' % (name,id))
        redirect_to(controller='boxbackup',action='index')
