# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
#
# This file is part of Satega. 
#
# aatdark <aatdark@gmail.com>
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
# Autor of this file:
# -> aatdark
#

import logging

from pylons import request, response, session, tmpl_context as c
from pylons.controllers.util import abort, redirect_to

from repoze.what.predicates import has_permission, in_group
from repoze.what.plugins.pylonshq import ActionProtector,ControllerProtector
import satega.lib.helpers as h

from satega.model.drive import DriveTools

from satega.lib.base import BaseController, render

log = logging.getLogger(__name__)

class DriveController(BaseController):
    @ActionProtector(in_group('admins'))
    def choose(self):
        c.drives = DriveTools.getDriveList()
        c.raid = DriveTools.getRaidDeviceObjects()
        c.maxCapacity = self._getMaxCapacity(c.drives)
        return render('/chooser.mako')
        
    @ActionProtector(in_group('admins'))
    def index(self):
        c.drives = DriveTools.getDriveList()
        c.drives_not_ready = DriveTools.getDriveList(False)
        c.raid = DriveTools.getRaidDeviceObjects()
        #get the max capacity in GB
        c.maxCapacity = self._getMaxCapacity(c.drives)
        return render('/drivelist.mako')
    def _getMaxCapacity(self,drives):
        currentMax = 0 
        for drive in c.drives:
            if drive.getSize() > currentMax:
                currentMax = drive.getSize()
                
        return currentMax


    @ActionProtector(in_group('admins'))
    def unmount(self,id):
        devPath = id.replace('-','/')
        part = DriveTools.getPartition(devPath)
        part.unmount()
        return self._success('drive' , 'index' , 'unmount successfull')
        
    @ActionProtector(in_group('admins'))
    def mount(self,id):
        devPath = id.replace('-','/')
        part = DriveTools.getPartition(devPath)
        mountpoint = part.mount()
        if mountpoint != None:
            return self._success('drive' , 'index' , 'mounted on %s' % (mountpoint,) )
        else:
            return self._error('drive' , 'index' , 'Error while mounting. Maybe no filesystem' )
        
    @ActionProtector(in_group('admins'))
    def format(self,id):
        devPath = id.replace('-','/')
        DriveTools.formatDevice(devPath)
        return self._success('drive' , 'index' , 'the device %s is not ready to use' % (devPath,) )
