# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
#
# This file is part of Satega. 
#
# aatdark <aatdark@gmail.com>
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
# Autor of this file:
# -> aatdark
#

import logging

from pylons import request, response, session, tmpl_context as c
from pylons.controllers.util import abort, redirect_to
from repoze.what.predicates import has_permission, in_group
from repoze.what.plugins.pylonshq import ActionProtector,ControllerProtector
import satega.lib.helpers as h
from satega.lib.base import BaseController, render

from satega.model.user import *
from satega.model.group import LdapGroup , LdapGroupControl
import formencode
from pylons.decorators import validate
log = logging.getLogger(__name__)


class GroupController(BaseController):
    @ActionProtector(in_group('admins'))
    def index(self):
        groups = LdapGroupControl.listGroups()
        c.groups = groups
        return render('/grouplist.mako')
    
    @ActionProtector(in_group('admins'))
    def addMember(self,id):
        c.users = LdapControl.listUsers()
        c.group = LdapGroupControl.loadGroup(id)

        return render('/groupaddmember.mako')
    
    @ActionProtector(in_group('admins'))    
    def submitaddmember(self,id):
        members = request.params.getall('selected')
        group = LdapGroupControl.loadGroup(id)
        if len(members) == 0 and id == 'admins' and 'cn=root,ou=Users,dc=satega,dc=local' in members:
            return self._error('group' , 'index' , 'The Admin group must at least contain one user')
        
        #overwrite the old values with the new ones
        group.members = members
        group.save()
        return self._success('group' , 'index' , 'Changed Group %s' % (id,))
    
    @ActionProtector(in_group('admins'))
    def submitcreate(self):
        name = request.params['name']
        LdapGroupControl.createGroup(name)
        
        redirect_to(controller='group', action='addMember' , id=name)
        #return self._success('group' , 'index' , 'successfully created %s' % (name,) )
    
    @ActionProtector(in_group('admins'))
    def remove(self,id):
        if id == 'admins':
            return self._error('/grouplist.mako' , 'can not remove admin group.')
        LdapGroupControl.removeGroup(id)
        return self._success('group' , 'index' ,  'successfully deleted %s' % (id,) )
        
    @ActionProtector(in_group('admins'))    
    def create(self):
        return render('/groupcreate.mako')
