# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
#
# This file is part of Satega.
#
# aatdark <aatdark@gmail.com>
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
# Autor of this file:
# -> aatdark
#

from satega.model.share import NfsShareTools
from pylons import request, response, session, tmpl_context as c
from pylons.controllers.util import abort, redirect_to

from repoze.what.predicates import has_permission, in_group
from repoze.what.plugins.pylonshq import ActionProtector,ControllerProtector
import satega.lib.helpers as h

from satega.model.drive import DriveTools
from satega.model.share import Share , ShareTools , UniqueShare
import formencode
from pylons.decorators import validate
from satega.lib.base import BaseController, render


import logging
log = logging.getLogger(__name__)

class SategaShare(formencode.Schema):
    messages = {
        'invalid': 'Please select an datastore',
    }
    allow_extra_fields = True
    filter_extra_fields = True
    if_key_missing = False
    comment = formencode.validators.String(not_empty=False)
    readonly = formencode.validators.StringBool(if_empty=False,not_empty=False)
    guest = formencode.validators.StringBool(if_empty=False,not_empty=False)
    part_choose = formencode.validators.MinLength(1,not_empty=True,messages = messages)

       
class SategaShareUpdate(SategaShare):
    name = formencode.validators.String(not_empty=True)

class SategaShareCreate(SategaShare):
    name = formencode.All(
            formencode.validators.String(not_empty=True),
            UniqueShare())
 


class ShareController(BaseController):
    @ActionProtector(in_group('admins'))
    def choose(self):
        c.drives = DriveTools.getDriveList()
        c.raid = DriveTools.getRaidDeviceObjects()
        c.maxCapacity = self._getMaxCapacity(c.drives)
        return render('/chooser.mako')

    @ActionProtector(in_group('admins'))
    def index(self):
        c.shares = ShareTools.get_all_share_objects()
        return render('/sharelist.mako')

    @ActionProtector(in_group('admins'))
    def submitdelete(self,id):
        if 'delete' in request.params:
            shareName = id
            sh  = ShareTools.get_share_object(shareName)
            # @type sh Share
            NfsShareTools.remove_nfs_share(sh.path) # remove nfs
            ShareTools.remove_samba_share(shareName)

            return self._success('share', 'index', 'removed %s' % (shareName,))
        redirect_to(controller='share',action='index')

    @ActionProtector(in_group('admins'))
    def mounthelp(self,id):
        from satega.lib.linuxhelper import Linux
        """
        shows an help page
        for users how to access shares
        @param id the sharename
        """
        c.server_name = hostname = Linux.get_hostname()
        c.share_name = id
        c.win_share_path = '\\\\%s\\%s' % (hostname,c.share_name )
        sh = ShareTools.get_share_object(id)
        c.share = sh
        return render('/share_mount_help.mako')
    @ActionProtector(in_group('admins'))
    def delete(self,id):
        """
        shows the are you sure form 
        """
        c.share_name = id
        return render('/sharedelete.mako')

    @ActionProtector(in_group('admins'))
    def change(self,id):
        shareName = id
        c.drives = DriveTools.getDriveList()
        c.raid = DriveTools.getRaidDeviceObjects()
        c.maxCapacity = self._getMaxCapacity(c.drives)
        c.update = True
        sh = ShareTools.get_share_object(shareName)
        # @type sh Share
        c.update_share_name = sh.get_name()
        
        defaults = { 'name' : sh.get_name() ,
                     'comment' : sh.comment,
                     'readonly' : int(not sh.writeable),
                     'guest' : int(sh.guest),
                     'part_choose' : sh.datastore.getDevPath(),
                    }
        return formencode.htmlfill.render(render('/sharecreate.mako'),defaults)

    @ActionProtector(in_group('admins'))
    @validate(schema=SategaShareUpdate(),form='change',post_only=False,on_get=True)
    def submitchange(self,id):
        shareName = id
        sh = ShareTools.get_share_object(shareName)

        name = self.form_result['name']
        readonly = self.form_result['readonly']
        guest = self.form_result['guest']
        devName = self.form_result['part_choose']
        comment = self.form_result['comment']
        sh.sharename = name
        sh.datastore = DriveTools.getPartition(devName)
        sh.writeable = not readonly
        sh.browseable = True
        sh.guest = guest
        sh.comment = comment
        sh.save()
        return self._success('share', 'index', 'Successfully updated share %s' % (name,))

    @ActionProtector(in_group('admins'))
    def create(self):
        c.drives = DriveTools.getDriveList()
        c.raid = DriveTools.getRaidDeviceObjects()
        c.maxCapacity = self._getMaxCapacity(c.drives)
        return render('/sharecreate.mako')

    @ActionProtector(in_group('admins'))
    @validate(schema=SategaShareCreate(),form='create',post_only=False,on_get=True)
    def submitcreate(self):
        name = self.form_result['name']
        readonly = self.form_result['readonly']
        guest = self.form_result['guest']
        devName = self.form_result['part_choose']
        comment = self.form_result['comment']
        sh = Share()
        sh.sharename = name
        sh.datastore = DriveTools.getPartition(devName)
        sh.loaded = False
        sh.writeable = not readonly
        sh.browseable = True
        sh.guest = guest
        sh.comment = comment
        sh.nfs_enabled = True
        ret = sh.save()
        if ret == True:
            return self._success('share', 'index', 'Successfully created share %s' % (name,))
        else:
            return self._error('share', 'index', 'an error occourd')

    def _getMaxCapacity(self,drives):
        currentMax = 0
        for drive in c.drives:
            if drive.getSize() > currentMax:
                currentMax = drive.getSize()

        return currentMax
