# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
# Svein Hansen <sveinh@gmail.com>
#
# This file is part of Satega (www.satega.org)
#
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
#

from subprocess import Popen, PIPE
#from support.errorcodes import EXEC_PROCESS_ERROR
#from support.sategafault import SategaFault
from time import time
import thread

class ShellProcess(object):
    """ Class to allow execution of shell commands from Python.

    """
    
    def __init__(self, cmd):
        self.cmd = cmd
        self.error = False
        self.stdout = None
        self.stderr = None
        self.returncode = None
        self.startTime = None
        self.endTime = None
        self.execTime = None
        self.input = ""

    def execute_async(self):
        """
        executes the command asyn (don't wait for it to finish)
        """
        thread.start_new_thread(self.execute,())
    def execute(self):
        """ Execute the command. Also measure time to execute.
        
        """
        self.error = False
        p = Popen(self.cmd, stdin=PIPE , stdout=PIPE, stderr=PIPE, shell=True, close_fds=True)
        self.startTime = time()
        (self.stdout, self.stderr) = p.communicate(self.input.encode())
        p.wait()
        self.endTime = time()
        self.execTime = self.endTime - self.startTime
        self.returncode = p.returncode
        if self.returncode != 0:
            self.error = True
    
def shLaunch(cmd,exception_on_fail=False):
    """ Utility function (shortcut) to the ShellProcess class.
    Ensure raising SategaFault exception upon error.
    
    """
    
    shellProcess = ShellProcess(cmd)
    shellProcess.execute()

    if shellProcess.error and exception_on_fail:
        raise Exception(0, 'Error reading output from command: ' + 
                          'Return code: ' + str(shellProcess.returncode) + ' ' +
                          'Command: ' + shellProcess.cmd + ' ' + 
                          'stderr: ' + shellProcess.stderr)
    return shellProcess

    
    
