# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
#
# This file is part of Satega (www.satega.org).
#
# aatdark <aatdark@gmail.com>
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
# Autor of this file:
# -> aatdark
#


from satega.lib.shellProcess import ShellProcess,shLaunch
from datetime import datetime

import os.path
import os
import re

import thread
import logging
log = logging.getLogger(__name__)

class Task(object):
    def __init__(self,name , args):
        self.thread = None
        self.args = args
        self.name = name
        self.start_time = None
        self.stop_time = None
    def get_status(self):
        pass

    def get_name(self):
        return self.name

    def get_progress(self):
        pass

    def get_start_time(self):
        return self.start_time

    def get_stop_time(self):
        return self.stop_time
    
    def is_working(self):
        """
        @returns if the task is working
        """
        if self.stop_time == None:
            return True
        return False
    def run(self):
        """
        method that is called to run
        """
        raise Exception('you can not use this class - its the parent')
    def start(self):
        self.thread = thread.start_new_thread(self.run,self.args)
        self.start_time = datetime.now()
    def stop(self):
        if self.thread != None:
            self.thread.stop()

class RemoveFilesTask(Task):
    def get_status(self):
        if self.stop_time == None:
            return 'Working'
        else:
            return 'Done'

    def run(self,path):
        """
        this method is the called method
        """
        if not os.path.isdir(path):
            log.error('params not directories from %s' % (path))
            return False
        log.info('Starting delete on  %s' % (path))
        sh  = ShellProcess('rm -Rf %s' % (path))
        sh.execute()
        log.info('delete files finished')
        self.stop_time = datetime.now()
        
class UpdateSystemTask(Task):
    def get_status(self):
        if self.stop_time == None:
            return 'Working'
        else:
            return 'Done'
    
    def run(self):
        """
        this method is the called method
        """
        
        log.info('Starting system update (apt-get)')
        sh = ShellProcess("/root/satega/systemupdate.sh")
        sh.execute()
        log.info('syste, updated')
        self.stop_time = datetime.now()

class UpdateSystemTask(Task):
    def get_status(self):
        if self.stop_time == None:
            return 'Working'
        else:
            return 'Done'

    def run(self):
        """
        this method is the called method
        """

        log.info('Starting system update (apt-get)')
        sh = ShellProcess("/root/satega/systemupdate.sh")
        sh.execute()
        log.info('System, updated')
        self.stop_time = datetime.now()


class ExecuteCmdTask(Task):
    def get_status(self):
        if self.stop_time == None:
            return 'Working'
        else:
            return 'Done'

    def run(self,cmd):
        """
        this method is the called method
        """
        log.info('Starting execution off %s' % (cmd))
        sh  = ShellProcess(cmd)
        sh.execute()
        log.info('cmd %s finished '%(cmd) )
        self.stop_time = datetime.now()

class MoveFilesTask(Task):
    def get_status(self):
        if self.stop_time == None:
            return 'Working'
        else:
            return 'Done'

    def run(self,source,dest):
        """
        this method is the called method
        """
        #TODO check with df if enough space is left
        if not os.path.isdir(source):
            log.error('params not directories from %s to %s' % (source,dest))
            return False
        log.info('Starting copy from %s to %s' % (source,dest))
        sh  = ShellProcess('mv %s %s' % (source,dest))
        sh.execute()
        log.info('move files finished')
        self.stop_time = datetime.now()

class TaskControl(object):
    __single = None # singleton
    def __init__(self):
        if TaskControl.__single != None:
            raise Exception('use get_instance')
        TaskControl.__single = self
        TaskControl.__single.tasklist = []

    @staticmethod
    def get_instance():
        if TaskControl.__single == None:
            t = TaskControl()
        return TaskControl.__single

    def cleanup(self):
        """
        removes all completed task from the list
        """
        newtasklist = []
        for task in self.tasklist:
            if task.is_working() == True:
                newtasklist.append(task)
        
        self.tasklist = newtasklist

    def register_task(self,task):
        """
        adds an task to the list of background tasks
        """
        self.tasklist.append(task)
        return True

    def remove_task(self,task):
        """
        removes an task from the list
        @param task an child of Task class
        @returns true if found
        """
        if task in self.tasklist:
            self.tasklist.remove(task)
            return True
        return False
    
    def get_tasks(self):
        return self.tasklist
