# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
#
# This file is part of Satega (www.satega.org).
#
# aatdark <aatdark@gmail.com>
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
# Autor of this file:
# -> aatdark
#

from logging import Logger
from satega.lib.linuxhelper import Linux
import os.path

from satega.lib.shellProcess import ShellProcess,shLaunch
from datetime import datetime
from webhelpers.html import literal

import os
import re

import thread
import logging
log = logging.getLogger(__name__)
import cPickle as pickle

class BackupAccount(object):
    def __init__(self,id):
        self.name = None # /etc/satega/boxbackup_mapping
        self.id = id
        self.used = (0,0)
        self.old_files = (0,0)
        self.deleted_files = (0,0)
        self.soft_limit = (0,0)
        self.hard_limit = (0,0)

    def free_in_gb(self):
        return float(self.hard() - self.used_in_gb())

    def convert_to_gb(self,source):
        """
        converts the source ('12 GB' into 12 with dest_unit='GB'
        """
        parts = source.split(' ')
        if parts[1].lower() == 'kb':
            return float(parts[0]) / 1024 / 1024
        if parts[1].lower() == 'mb':
            return float(parts[0]) / 1024
        if parts[1].lower() == 'gb':
            return float(parts[0])
        if parts[1].lower() == 'tb':
            return float(parts[0]) * 1024
        raise Exception('can not convert %s' % (source))

    def used_html(self):
        """
        @returns an literal 12&nbsp;GB
        """
        return literal(self.used[0].replace(' ' , '&nbsp;'))

    def used_in_gb(self):

        return self.convert_to_gb(self.used[0])

    def used_percent(self):
        """
        @returns  the percent of the used value
        """
        return int(self.used[1].split('%')[0])
    def soft(self):
        """
        @returns SOFT_LIMIt in GB
        soft.self_limit format : ('12 GB','66%')
        first : absolute second : relative
        """
        return float(self.soft_limit[0].split(' ')[0])
    def hard(self):
        return float(self.hard_limit[0].split(' ')[0])

class AccountNameMapping(object):
    """
    provides some basic methods to
    map the id of the backup account
    to an rememberable name
    """
    @staticmethod
    def get_name_from_id(id):
        """
        returns the account name
        stored in /etc/satega/boxbackup_mapping
        @param id will be converted to int
        @return name or None if not found
        """
        id = int(id)
        dict = AccountNameMapping.get_dict()
        if id in dict:
            return dict[id]
        else:
            return None
    @staticmethod
    def set_and_save(id,name):
        """
        directly changes the file
        @param id the id of the backup account ( is normalized as int)
        @param name given name
        """
        id = int(id)
        log.info('%s %s' % (id,name))
        dict = AccountNameMapping.get_dict()
        dict[id] = name
        AccountNameMapping.save_dict(dict)
        
    @staticmethod
    def save_dict(dict):
        filename = '/etc/satega/boxbackup_mapping'
        if not os.path.isfile(filename):
            shLaunch('touch %s' % (filename))
        output = open(filename,'w')
        pickle.dump(dict, output)
        output.close()
    @staticmethod
    def get_dict():
        if not os.path.isfile('/etc/satega/boxbackup_mapping'):
            AccountNameMapping.save_dict({})
        pkl_file = open('/etc/satega/boxbackup_mapping','r')
        dict = pickle.load(pkl_file)
        pkl_file.close()
        return dict
class Boxbackup(object):

    @staticmethod
    def get_account_object(id):
        """
        @returns the object for one given id
        """
        data = Boxbackup.get_stats_for_client(id)
        ac = BackupAccount(id)
        ac.name = AccountNameMapping.get_name_from_id(id)
        ac.used = data['Used']
        ac.old_files = data['Old files']
        ac.deleted_files = data['Deleted files']
        ac.soft_limit = data['Soft limit']
        ac.hard_limit = data['Hard limit']
        return ac

    @staticmethod
    def get_account_objects():
        """
        @returns all the accounts on the system
        in BackupAccount objects
        """
        ret = []
        for id in Boxbackup.get_account_list():
            ac = Boxbackup.get_account_object(id)
            ret.append(ac)
        return ret


    @staticmethod
    def get_account_list():
        """
        returns an list with ids
        """
        ret = []
        file = open('/etc/boxbackup/bbstored/accounts.txt','r')
        for line in file.readlines():
            parts = line.split(':')
            ret.append(int(parts[0]))
        return ret


    @staticmethod
    def next_account_id():
        """
        returns the next free account number
        """
        ids = Boxbackup.get_account_list()
        currentMax = -1
        for id in ids:
            if id > currentMax:
                currentMax = id
        return currentMax+1

    @staticmethod
    def create_client_certs(id):
        """
        @param id the id of the account
        @param hostname the hostname important for openssl cert
        """
        conf_dir = '/etc/satega/boxbackup/client/%s' %(id)

        #create folders rekursiv
        shLaunch('mkdir -p %s' % (conf_dir))
        servername = Linux.get_hostname()
        certs = ShellProcess('bbackupd-config \
            %(conf_dir)s \
            %(mode)s  \
            %(accountno)s \
            %(server)s \
            %(tmpdir)s \
            %(backupdirs)s ' % \
                {'conf_dir' : conf_dir,
                 'mode' : 'snapshot',
                 'accountno' : id,
                 'server' : servername,
                 'tmpdir' : '/tmp',
                 'backupdirs' : '/etc'
                })
        certs.execute()
        #print certs.stdout , certs.stderr
        csr_file = conf_dir + '/bbackupd/%s-csr.pem' % (id)
        ca_path = '/etc/boxbackup/ca'
        #sign the file
        sign = ShellProcess('bbstored-certs  %s sign %s' % (ca_path , csr_file) )
        sign.input = u'yes%s' %(os.linesep)
        sign.execute()
        #print sign.stdout , sign.stderr
        
        #now you have to copy the following files to the client
        # /etc/boxbackup/ca/clients/1-cert.pem
        # /etc/boxbackup/ca/roots/serverCA.pem
        return True

    @staticmethod
    def get_stats_for_client(id):
        """
        returns the status for the given client
        format ('key' : (absolute value , percent) )
        """
        stats = ShellProcess('bbstoreaccounts info %s' % (id))
        stats.execute()
        ret = {}
        for line in stats.stdout.splitlines():
            line = line.strip()

            #these lines are useless for us
            if line.startswith('Account') or line.startswith('Last object') or line.startswith('Client store'):
                continue
            parts = line.split(':')
            option = parts[0]
            parts2 = parts[1].split(',')
            percent = parts2[2].strip().split(' ')[0]
            ret[option] = (parts2[1].strip() , percent)
        return ret


    @staticmethod
    def get_filename_for_cert(id,typ):
        """
        @returns the correct filename for the givven cert typ
        """
        if typ == 'raw':
            return '%(id)s-FileEncKeys.raw' % {'id' :id }
        if typ == 'cert':
            return '%(id)s-cert.pem' % {'id' :id }
        if typ == 'key':
            return '%(id)s-key.pem'% {'id' :id }
        if typ == 'ca':
            return 'serverCA.pem'
        return None

    @staticmethod
    def get_certs_for_client(id):
        """
        returns the keyfiles the backupclient needs to work
        KeysFile = /etc/satega/boxbackup/clients/1/bbackupd/1-FileEncKeys.raw
        CertificateFile = /etc/boxbackup/ca/clients/1-cert.pem
        PrivateKeyFile = /etc/satega/boxbackup/client/1/bbackupd/1-key.pem
        TrustedCAsFile = /etc/boxbackup/ca/roots/serverCA.pem

        The client needs all of them
        @returns ('key' :val , 'raw':val , 'cert':val  , 'ca' : val)
        """
        if id not in Boxbackup.get_account_list():
            raise Exception('client id not found')
        ret = {}
        raw = open('/etc/satega/boxbackup/client/%(id)s/bbackupd/%(id)s-FileEncKeys.raw' % {'id' :id } ,'r')
        ret['raw'] = raw.read()
        raw.close()

        cert = open('/etc/boxbackup/ca/clients/%(id)s-cert.pem' % {'id' :id } , 'r')
        ret['cert'] = cert.read()
        cert.close()

        key = open('/etc/satega/boxbackup/client/%(id)s/bbackupd/%(id)s-key.pem' % {'id' :id } , 'r')
        ret['key'] = key.read()
        key.close()

        ca = open('/etc/boxbackup/ca/roots/serverCA.pem' , 'r')
        ret['ca'] = ca.read()
        ca.close()

        return ret

    @staticmethod
    def update_account(id , computer_name , soft_limit,hard_limit):
        """
        updates the limits and/or computer name
        id can not be changed
        @param id the account id to change
        @param computer_name the new name for the account
        @param soft_limit the new softlimit in GB
        @param hard_limit new hard limit in GB
        """
        if soft_limit > hard_limit:
            raise Exception('soft limit must be smaller than hard_limit')
        AccountNameMapping.set_and_save(id, computer_name) # add the mapping entry
        cmd = 'bbstoreaccounts setlimit %s %dM %dM' % (id,int(soft_limit*1024) , int(hard_limit*1024))
        sp = shLaunch(cmd)
        # @type sp ShellProcess
        if sp.returncode != 0:
            # @type log: Logger
            log.warn('update of limits failed', (id,sp.stderr), kwargs)
        return True


    @staticmethod
    def create_account_and_certs(computer_name , soft_limit,hard_limit,id=None):
        """
        creates an new account for an maschine to backup
        if you want to create the certs call create_client_certs
        also creates the cerst for the client BoxBackup,create_client_certs(id)
        @param computer_name unused
        @param soft_limit the soft limit in GB
        @param hard_limit hard limit in GB
        @param id: the id to use. If None the next number is taken
        @returns the assigned id
        """
        if id == None:
            id = Boxbackup.next_account_id()
        cmd = 'bbstoreaccounts create %s 0 %sM %sM' % (id,soft_limit*1024 , hard_limit*1024)
        shLaunch(cmd)
        AccountNameMapping.set_and_save(id, computer_name) # add the mapping entry
        Boxbackup.create_client_certs(id)
        return id

    @staticmethod
    def get_raidfile_path():
        """
        returns the path on where
        boxbackup stores the data
        @returns path of the boxbackup backups
        """
        file = open('/etc/boxbackup/raidfile.conf','r')
        for line in file.readlines():
            if line.strip().startswith('Dir0'):
                parts = line.split('=')
                return parts[1].strip()

    @staticmethod
    def setup_raidfile(path):
        """
        setup the place where boxbackup stores the backups
        @param path the folder to store the backups
        """
        raidfile = '/etc/boxbackup/raidfile.conf'
        if os.path.exists(raidfile):
            os.remove(raidfile)
        shLaunch('raidfile-config /etc/boxbackup 4096 %s' % (path))
        return True
    
    @staticmethod
    def start_server():
        """
        starts the boxbackup server
        """
        pass
    @staticmethod
    def delete_account(id):
        """
        removes an existing account
        """
        cmd = 'bbstoreaccounts delete %s yes' % (id)
        shLaunch(cmd)
        return True
