# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
#
# This file is part of Satega (www.satega.org). 
#
# aatdark <aatdark@gmail.com>
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
# Autor of this file:
# -> aatdark
#


from satega.lib.shellProcess import ShellProcess,shLaunch
import os
import re

import logging
log = logging.getLogger(__name__)

class DriveTools(object):
    """
    this class provides static commands
    to manage / format / mount
    devices
    """
    @staticmethod
    def getDeviceFiles():
        """
        @return all linux dev files
        like /dev/sda to /dev/sdz
        """
        cmd = shLaunch("ls /dev/ | grep -E '^s[a-z]{2}$'")
        outputString = cmd.stdout
        devices = outputString.split()
        
        ret = []
        for devName in devices:
            path = '/dev/' + devName
            if not os.path.isdir(path):
               if DriveTools.checkIfDevIsPartOfRaid(path) == False:
                   ret.append(path)
        
        return ret
        """
        deletetions = 0
        for i in range(len(devices)):
            if os.path.isdir("/dev/"  + devices[i-deletetions]) == False:
                devices[i-deletetions] = "/dev/"  + devices[i-deletetions]
            else:
                del devices[i-deletetions]
                deletetions = deletetions+1
        return devices
        """
    
    @staticmethod
    def getRaidDeviceFiles():
        """
        @return all the linux software raid devices like /dev/md0
        """
        cmd = ShellProcess("ls /dev/ | grep -E '^md[0-9]{1}$'")    
        cmd.execute()
        outputString = cmd.stdout
        if cmd.error == True:
            return []
        devices = outputString.split()
        for i in range(len(devices)):
            devices[i] = "/dev/"  + devices[i]
        return devices
    
    @staticmethod
    def getRaidDeviceObjects():
        """
        @returns objects of partition
        """
        ret = []
        for devpath in DriveTools.getRaidDeviceFiles():
            data = DriveTools.getParameterofRaidDev(devpath)
            part = Partition()
            part.mountpoint = DriveTools.getMountPointForPartition(devpath)
            part.devname = devpath
            part.size = ( data['size'] , 'GB')
            part.filesystem = ''
            
            if part.mountpoint != None:
                storagestat = DriveTools.getAttribsofMountedDevices(devpath)
                part.used = storagestat['used']
                part.free = storagestat['free']
                
            ret.append(part)
        return ret
    @staticmethod
    def getMapperDeviceFiles():
        """
        @return all the linux dmraid software raids in /dev/mapper
        """
        if os.path.exists('/dev/mapper/') == False:
            return []
        cmd = shLaunch("ls /dev/mapper/ | grep -E '^[a-z_]*$'")
        outputString = cmd.stdout
        devices = outputString.split()
        
        if "control" in devices:
            devices.remove("control")
        for i in range(len(devices)):
            devices[i] = "/dev/mapper/"  + devices[i]
        return devices
    
    @staticmethod
    def getFreeUsedDiskSpace(mountPoint):
        """
        gets the freediskspace for an mounted partition
        @return an dict with free and used as keys
        """
        # is this realy needed ? TODO
        cmd = ShellProcess("df " +mountPoint)
        cmd.execute()
        #Filesystem        1K-blocks     Used Available Use% Mounted on
        #/dev/sda1             2924572   1379108   1398072  50% /
        # parts[0]               parts[1]    parts[2]  parts[3] ...
        #print cmd.stdout
        parts = cmd.stdout.split("\n")[1].split()
        return { 'free' : int(parts[3]) , 'used' : int(parts[2])}
    
    @staticmethod
    def getMountPointForPartition(devName):
        """
        gets the mount point or returns None if
        the device is not mounted
        @return None if not mounted
        """
        cmd = ShellProcess("cat  /etc/mtab | grep " + devName)
        cmd.execute()
        parts = cmd.stdout.split()
        if len(parts) > 1:
            return parts[1]
        else:
            return None
    
    @staticmethod
    def getParameterofRaidDev(devName):
        """
        collects the data (size,raidlevel) with the mdadm command
        @returns dict ('size' , 'raidlevel') units in GB
        """
        cmd = ShellProcess('mdadm --detail %s' % (devName,))
        cmd.execute()
        c = re.compile("Array Size : (\d*)")
                      # ([0-9.]*)([A-Za-z]*)")
        m = c.search(cmd.stdout)
        
        size = float(m.groups()[0])
        c = re.compile("Raid Level : raid(\d*)")
                      # ([0-9.]*)([A-Za-z]*)")
        m = c.search(cmd.stdout)
        raidlevel = int(m.groups()[0])
        
        return { 'size' : size/1024.0/1024 , 'raidlevel' : raidlevel }
        
    @staticmethod
    def getPartitionsForDevice(devName):
        """
        returns an list of dict with information about the partitions
        @param : devName the DrivePath like /dev/sda
        @return: a list ( of dict ( 'devname','size','filesystem' , 'mountpoint' , 'free' , 'used'))
        """
        cmd = ShellProcess("parted -s "+ devName + " print")
        cmd.execute()
        if cmd.error == True:
            return []
        outputString = cmd.stdout
        
        lines = outputString.split('\n')
        ret = []
        lines = lines[6:]
        for line in lines:
            if line == "":
                break
            parts = line.split()
            no = parts[0]
            c = re.compile("([0-9.]*)([A-Za-z]*)")
            m = c.match(parts[3])
            unit = m.groups()[1]
            size = float(m.groups()[0])
                
            if len(parts)>5:
                fs = parts[5]
            else:
                fs =""
            mountpoint = DriveTools.getMountPointForPartition(devName+no)
            if mountpoint != None:
                data = DriveTools.getAttribsofMountedDevices(devName+no)
                freeusedDiskspace = { 'free' : data['size']-data['used'], 'used' : data['used'] }
                #freeusedDiskspace = DriveTools.getFreeUsedDiskSpace(mountpoint)
            else:
                freeusedDiskspace = { 'free' : 0, 'used' : 0 }
            if parts[4]!= "extended" and fs != 'linux-swap':
                ret.append({   'devname' : devName+no ,
                              'size' :  (size,unit) ,
                              'filesystem' : fs,
                              'mountpoint' : mountpoint,
                              'free' : freeusedDiskspace['free'],
                              'used' : freeusedDiskspace['used']})
        return ret
    
    @staticmethod
    def checkIfDevIsPartOfRaid(devName):
        """
        checks if the given device is part of a raid (md, or dmraid)
        @param devName: the devicepath like /dev/sda
        @return true if part otherwise false
        
        $dmraid -r
        /dev/sdb: isw, "isw_gaiieahb", GROUP, ok, 488397166 sectors, data@ 0
        /dev/sda: isw, "isw_gaiieahb", GROUP, ok, 488397166 sectors, data@ 0
        
        $cat /proc/mdstat 
        Personalities : [linear] [multipath] [raid0] [raid1] [raid6] [raid5] [raid4] [raid10] 
        md0 : active raid1 sdc1[0] sdb1[1]
             488383936 blocks [2/2] [UU]
        unused devices: <none>
        """
        cmd = ShellProcess("dmraid -r")
        cmd.execute()
        if devName in cmd.stdout:
            return True
        
        # extract the last part from the path
        #/dev/name => name
        parts = devName.split('/')
        shortDevName = parts[len(parts)-1]
        
        cmd = ShellProcess("cat /proc/mdstat")
        cmd.execute()
        if shortDevName in cmd.stdout:
            return True
        return False
    
    @staticmethod
    def getDeviceCapacity(devName):
        """
        returns the capacity (total size) of the device
        @return (size , unit)  for example : ( 100 , 'MB') (unit is always 'MB')
        """
        cmd = ShellProcess("parted -s "+ devName + " print")
        cmd.execute()
        if cmd.error == True:
            return []
        outputString = cmd.stdout
        lines = outputString.split('\n')
        line = lines[1]
        parts = line.split()
        
        c = re.compile("([0-9.]*)([A-Za-z]*)")
        m = c.match(parts[2])
        unit = m.groups()[1]
        capacity = float(m.groups()[0])
        
        if unit == "GB":
            capacity = capacity  * 1024
            unit= "MB"
        return (capacity ,unit)
    
    @staticmethod
    def formatDevice(devName):
        """
        this method calls sfdisk and formats the whole disk !!
        ALL DATA IS LOST
        this inizializes the device withhin satega.
        """
        print "formatDevice" , devName
        purgeData=ShellProcess("dd if=/dev/zero of="+devName+" bs=1KB count=1")
        purgeData.execute()
        cmd = ShellProcess("sfdisk "+ devName)
        cmd.input = "0\n0\n0\n"
        cmd.execute()
        print cmd.stdout , cmd.stderr
        cmd = ShellProcess("mkfs.ext3 "+ devName+"1")
        cmd.execute()

    @staticmethod
    def formatFilesystem(devName,filesystem):
        """
        Formats the given devName Partion with the given filesystem (for example filesystem:ext3
        @param devName : the device name like /dev/sda1
        @param filesystem: the filesystem type: ext3 , ..
        """       
        #self.logger.debug("mkfs. " + filesystem + " " + devName)
        cmd = ShellProcess("mkfs." + filesystem + " " + devName)
        cmd.execute()
        print cmd.stderr
        print cmd.stdout
        return True
    
    @staticmethod
    def unmountDevice(mountpoint):
        """
        umounts the given device with the given mountpoint
        @param mountPoint: the mountpoint to unmount
        """
        cmd = shLaunch("umount " + mountpoint)
        return True
    
    @staticmethod
    def mountDevice(devName, mountpoint):
        """
        umounts the given device with the given mountpoint
        @param devName: the device to mount like /etc/sda1
        @param mountpoint: "" => autogenerated mountpoint, otherwise path
        @return the mountpoint or exception
        """
        if mountpoint == "":
            #autoset mountpoint
            parts = devName.split("/")
            name = "".join(parts)
            mountpoint  ="/mnt/" + name
            if not os.path.isdir(mountpoint):
                os.mkdir(mountpoint)
        cmd = ShellProcess("mount " + devName + " " + mountpoint)
        cmd.execute()
        if (cmd.returncode == 32 ):
            # mount: you must specify the filesystem type
            # most common cause > no filesystem
            log.error("Mounterror 32 :%s %s"% ( devName , mountpoint))
            return None
        return mountpoint
    
    @staticmethod
    def getPartition(devName):
        if devName.startswith('/dev/md'):
            #raid device > use the getRaidDeviceObjects
            raids = DriveTools.getRaidDeviceObjects()
            for raid in raids:
                if devName == raid.getDevPath():
                    return raid
            #not found
            return None
        drivedev = devName[0:len(devName)-1]
        parts  =  DriveTools.getPartitionsForDevice(drivedev)
        found = None
        for row in parts:
            if row['devname'] == devName:
                found = row
                
        if found == None:
            return False
        partition = found
        part = Partition()
        part.mountpoint = partition['mountpoint']
        part.devname = partition['devname']
        part.size = partition['size']
        part.filesystem = partition['filesystem']
        part.used = partition['used']
        part.free = partition['free']
        return part
    

    @staticmethod
    def getDriveList(only_ready_to_use=True):
       """
       return the Drive Objects
       @param only_ready_to_use : default true: Hide the drives that are unformatted 
       Not Ready devices are for example : unpartionized devices
       """
       #TODO DriveTools.getRaidDeviceFiles() +
       devs  =   DriveTools.getDeviceFiles() + DriveTools.getMapperDeviceFiles() #+ DriveTools.getDevicesFromConfigFile()
       ret  = []
       for device in devs:
            disk = Disk()
            disk.name = ''
            disk.devpath = device
            disk.size = DriveTools.getDeviceCapacity(device)
            parts  =  DriveTools.getPartitionsForDevice(device)
            if only_ready_to_use == True and len(parts) == 0:
                continue
            if only_ready_to_use == False and len(parts) > 0:
                continue
            for partition in parts:
                part = Partition()
                part.mountpoint = partition['mountpoint']
                part.devname = partition['devname']
                part.size = partition['size']
                part.filesystem = partition['filesystem']
                part.used = partition['used']
                part.free = partition['free']
                disk.partitions.append(part)
            
            ret.append(disk)
       return ret
    @staticmethod
    def autoMountAllParts():
        """
        tries to mount all unmounted partitions
        """
        drives = DriveTools.getDriveList()
        for drive in drives:
            for part in drive.getPartitions():
                part.mount()
        
        for raid in DriveTools.getRaidDeviceObjects():
            raid.mount()
                
    @staticmethod
    def getAttribsofMountedDevices(devName):
        """
        returns the freespace and the usedspace and the mountpoint
        this is more accurate than the partition size, because this
        method returns the netto space ( and not the theoretic capacity)
        @param devName: the device name like /dev/sda1
        @return False if not mounted , otherwise dict with values
        (size,used,percent,mountpoint,devname)
        units in GB
        
        """
        #never used till now
        cmd = shLaunch("df  " + devName)
        outputString = cmd.stdout
        attributes =  outputString.split()[7:]
        # devices that are 'mounted on /dev' are not mountable (ignoring)
        if attributes[5].startswith("/dev") == True:
            return False
        
        ret = {}
        ret['size'] = float ( attributes[1] ) /1024/1024 
        ret['used'] = float ( attributes[2] ) /1024/1024 
        ret['free'] = float ( attributes[3] ) /1024/1024 
        ret['percent'] = attributes[4]
        ret['mountpoint'] = attributes[5]
        ret['devname'] = devName
        return ret
        
class Disk(object):
    def __init__(self):
        self.name = ""
        self.devpath = ''
        self.state = 0
        self.partitions = []
        self.size = 0
    
    def getName(self):
        return self.name
    
    def __repr__(self):
        return '@Disk:%s' % (self.devpath)
    def __eq__(self,b):
        """
        allow group objects to be comparted to each other
        and to strings
        """
        if isinstance(b,basestring):
            if self.devpath == b:
                return True
        return False
    
    def isRaid(self):
        if self.name.startswith('/dev/md') == True:
            return True
        return False
    
    def getDevPath(self):
        return self.devpath
    
    def getPartitions(self,only_mounted=False):
        if only_mounted:
            ret = []
            for part in self.partitions:
                # @type part Partition
                if part.mountpoint != None:
                    ret.append(part)
            return ret
        else:
            return self.partitions
    
    def getSize(self):
        """
        @return the size in GB (if useable). otherwise 0 is returned
        """
        if len(self.size) == 0:
            return 0
        
        if self.size[1] == 'MB':
            return self.size[0] / 1024
        return self.size[0]

class Partition(object):
    def __init__(self):
        self.mountpoint = None
        self.usable = False
        self.devname = ''
        self.used = 0
        self.free = 0
        self.size = None
        self.filesystem = None
    
    def __repr__(self):
        return '@Partition: %s %s' % (self.devname , self.mountpoint)
    def getDevName(self):
        """
        @return the devPath without the starting /dev/
        """
        res = self.devname.split('/')
        return res[len(res)-1]
        
    def getDevPath(self):
        return self.devname
    
    def isMounted(self):
        """
        @return True if mounted
        """
        if self.mountpoint != None:
            return True
        else:
            return False
    def getMountPoint(self):
        return self.mountpoint
            
    def getSizeInPercent(self):
        """
        returns the size in %
        only possible when mounted
        """
        if self.mountpoint != None:
            return round ( 100 * self.used / (self.used +self.free) ,2)
        else:
            return 100
        
    def getUsed(self):
        if self.mountpoint != None:
            return self.used
        else:
            if self.size[1] == 'MB':
                return self.size[0] / 1024
            else:
                return self.size[0]
    
    def getFree(self):
        """
        @return : the freesize in GB
        """
        return self.free
    
    def getSize(self):
        if self.mountpoint != None:
            return self.getFree() + self.getUsed()
        else:
            if self.size[1] == 'MB':
                return self.size[0] / 1024
            else:
                return self.size[0]
        
    def getFilesystem(self):
        return self.filesystem
    def format(self,filesystem):
        """
        format the partition with the given filesystem
        @param filesystem: the filesystem (ext3 , ..)
        """
        pass
    def mount(self,mountpoint=''):
        """
        mount on the given mountpoint
        @param mountpoint : default is autogenerated mountpoint
        @return False if not mounted
        """
        if self.mountpoint != None or self.filesystem == 'linux-swap':
            return False
        log.debug('mount %s on %s'% (self.devname ,self.mountpoint ))
        res = DriveTools.mountDevice(self.devname,mountpoint)
        if res != False:
            if mountpoint == '':
                self.mountpoint = res
            else:
                self.mountpoint = mountpoint
        return res
        
        
    def unmount(self):
        """
        unmounts the partition
        @return True if success otherwise ( not mounted ) False
        Troubles : Exception
        """
        if self.mountpoint == None or self.filesystem == 'linux-swap':
            return False
        DriveTools.unmountDevice(self.mountpoint)
        self.mountpoint = None
    
class DriveManagment(object):

    def __getAttributes(self,devName):
       """
       returns the freespace and the usedspace and the mountpoint
       """
       cmd = shLaunch("df  " + devName)
       outputString = cmd.stdout
       attributes =  outputString.split()[7:]
       if attributes[5].startswith("/dev") == True:
            return False
       attributes.append(devName)
       return attributes[2:]
    

    
