# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
#
# This file is part of Satega. 
#
# aatdark <aatdark@gmail.com>
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
# Autor of this file:
# -> aatdark
#

import logging
import sqlite3
import traceback
from sategamodule import SategaModule
from service import Servicelist

CD_DB_NOT_SUPPORTED_YET = 23001
CD_DB_NAME_NOT_UNIQUE = 23002


class SategaCd(SategaModule):
    """SategaCd

    """    
    
    def __init__(self):
        SategaModule.__init__(self)
        self.db_file = ''

    # override methods from parentclass
    def getName(self):
        return "Cd Module"
    
    def disable(self):
        return True #at the moment not possible
    
    def enable(self):
        return True
    
    def isEnabled(self):
        return True
    
    def initialize(self):

        self.db_file = self.module_config.get("main", "dbFile")

        # checks if the cd modul is installed correctly
        connection = sqlite3.connect(self.db_file, isolation_level=None)
        try:
            #try to search.
            self.search('%%', [], - 10)
            return True        
        except Exception, s:
            self.logger.error(s)
            return False
        finally:
            connection.close()

        return True
        

    # end override
    def getDbPath(self):
        """
        returns the path to the database file
        """
        return self.db_file
        
        
    def initializeDatabase(self):
        """
        installs the cd managment system
        most important action is to create the databasefile
        """
        try:
            connection = sqlite3.connect(self.db_file, isolation_level=None)
            cursor = connection.cursor()
            cursor.execute("BEGIN TRANSACTION;")
            cursor.execute("DELETE FROM cd_tag;")
            cursor.execute("DELETE FROM tag;")
            cursor.execute("DELETE FROM tag_typ;")
            cursor.execute("DELETE FROM cd;")
            cursor.execute("COMMIT;")

        finally:
            cursor.close()
            connection.close()
        location = self.addTagTyp(None, "Location", "111222333")
        medium = self.addTagTyp(None, "Medium", "333111222")
        
        #tag_id,name,rgb_color, tag_typ_id):
        self.addTag(None, "Celler", "no desc", "", location)
        self.addTag(None, "Cupboard", "no desc", "", location)        
        
        self.addTag(None, "DVD", "no desc", "", medium)        
        self.addTag(None, "CD", "no desc", "", medium)        
        self.addTag(None, "BlueRay", "no desc", "", medium)
        #sqlite3 /var/lib/satega/cd-management.db  < sql.sql
        

    
    def __createWhereINString(self, count):
        """
        creates an string for the WHERE IN command of the database
        for example WHERE cd_id IN ( 1,3,12)
        but it returns just (%,%,%)
        """
        part = ["?"] * count
        retString = "(" + ','.join(part) + ")"
        return retString
    
    def getAllCdsWithTags(self):
        """
        gets all CDS that are known to the database
        with all of their tags
        """
        return self.search('%%', [])
    
    def search(self, likeName, filter_tags, filter_cd_id=None):
        """
        this ist the main method to search for a cd

        Parameter:
        
        likeName 
            the Like String for the database. Is matched against CD_Name and Description
            For example: '%%' will get all cds. All startings with Ubuntu: 'Ubuntu%'

        filter_tags
            is an array with tag_ids. the matched cd, contain ALL search for tags.
            
        filter_cd_id
            like load an cd, with all the tags. Only used if != None
        Returns:
        Array of CDs.
        Each CD is array(cddata) , array(tags)
        [
            (
                (9L, 'ubuntu', 'desc'),   << CD NUMBER 9
                [
                    (1L, 'cd', None, 'media',1,''),  << HAS 3 TAGS TAG ( TAGID , TAGNAME , TAGDESC , TAG_TYPE_NAME ,  TAG_TYP_ID ,   Color )
                    (2L, 'dvd', None, 'media', 1,''), 
                    (5L, 'celler', None, 'location', 2,'')
                ]
            ), 
            (
                (10L, 'ubuntu', 'desc'),  << CD NUMBER 10 WITH ONE TAG
                [
                    (2L, 'dvd', None, 'media', 1,'')
                ]
            ), 
            (
                (11L, 'ubuntu', 'desc'),  << NEXT CD
                [
                    (1L, 'cd', None, 'media',1, ''), 
                    (2L, 'dvd', None, 'media', 1,'')
                ]
            )
        ]
        """
        if likeName == "":
            likeName = "%%"
        #php xml rpc fix
        if filter_cd_id == "":
            filter_cd_id = None
        
        connection = sqlite3.connect(self.db_file, isolation_level=None)
        cursor = connection.cursor()
        try:
            sqlString = """
             select 
                cd.cd_id , 
                cd.cd_name , 
                cd.cd_desc , 
                tag.tag_id , 
                tag.tag_name , 
                tag.tag_desc ,  
                tag_typ.name as tag_typ_name , 
                tag_typ.tag_typ_id,
                tag.color_rgb as tagrgb , 
                tag_typ.color_rgb as typrgb
            from cd 
                    left join cd_tag on cd.cd_id = cd_tag.cd_id 
                    left join tag on cd_tag.tag_id = tag.tag_id 
                    left join tag_typ on tag_typ.tag_typ_id = tag.tag_typ_id
                        WHERE 
                            cd.cd_id=? AND 
                            (cd.cd_name LIKE ? OR cd.cd_desc LIKE ?) 
                                ORDER BY cd.cd_id , tag.tag_typ_id , tag.tag_id"""

            #remove WHERE part if the cd_id is not our goal
            if filter_cd_id == None:
                sqlString = sqlString.replace("cd.cd_id=? AND", "")                
                cursor.execute(sqlString, (likeName, likeName))
            else:
                cursor.execute(sqlString, (filter_cd_id, likeName, likeName))
            ret = []
            first = cursor.fetchone()
            while (first != None) :
                needed_tag_ids = filter_tags[:]
                cd = (first[0:3], [])
                row = first
                while (True) :
                    tag_id = row[3]
                    if tag_id in needed_tag_ids:
                        del needed_tag_ids[needed_tag_ids.index(tag_id)]
                    TagRGB = row[8]
                    TypRGB = row[9]
                    tag = row[3:8]
                    # the tag rgb overrules the tagtyp rgb
                    finalRGB = ''
                    if TagRGB == "":
                        finalRGB = TypRGB
                    else:
                        finalRGB = TagRGB
                    tag = tag + (finalRGB,)
                    if tag_id != None:
                        cd[1].append(tag)
                    
                    row = cursor.fetchone()
                    #next jump
                    if row == None or (row[0] != first[0]):
                        break
                if len(needed_tag_ids) == 0:
                    #all search conditions are met
                    ret.append(cd)
                first = row
            return ret
        except Exception:
            raise    
        finally:
            cursor.close()
            connection.close()
            
    def getCd(self, cd_id):
        """
        same return typ as search fkt
        """
        cds = self.search('%%', [], cd_id)
        if len(cds) == 0:
            return None
        else:
            return cds[0]
    
    def getTagList(self):
        """gets all the tags that are know to the
        database
    +--------+------------------+----------+----------+------------+-----------+-----------+
    | tag_id | tag_name         | tag_desc | name     | tag_typ_id | color_rgb | color_rgb |
    +--------+------------------+----------+----------+------------+-----------+-----------+
    |      3 | blueray          |          | media    |          1 |           |           | 
    |      2 | dvd              |          | media    |          1 | 240240010 |           | 
    |      1 | cd               |          | media    |          1 | 255010000 |           | 
    |      5 | celler           |          | location |          2 |           |           | 
    |      4 | my room cupboard |          | location |          2 |           |           | 
    +--------+------------------+----------+----------+------------+-----------+-----------+

        5 rows in set (0.00 sec)
        
        RETURNS Array (tag_typ) which contains tags:
        (
            (TAGdata, TAGdata, TAGdata, ..),
            (TAGdata, TAGdata, TAGdata, ..),
        )
        """
        connection = sqlite3.connect(self.db_file, isolation_level=None)
        cursor = connection.cursor()
        try:
            cursor.execute("""
                 select tag.tag_id , tag.tag_name , tag.tag_desc ,  tag_typ.name , tag.tag_typ_id , tag.color_rgb , tag_typ.color_rgb  
                     from tag 
                         join tag_typ on tag.tag_typ_id = tag_typ.tag_typ_id
                             order by tag.tag_typ_id , tag.tag_id""")
             
            ret = []
            first = cursor.fetchone()
            while (first != None) :
                tag_typ = []
                row = first
                while (True) :
                    TagRGB = row[5]
                    TypRGB = row[6]
                    tag = row[0:5]
                    # the tag rgb overrules the tagtyp rgb
                    finalRGB = ''
                    if TagRGB == "":
                        finalRGB = TypRGB
                    else:
                        finalRGB = TagRGB
                    tag = tag + (finalRGB,)
                    tag_typ.append(tag)
                    
                    row = cursor.fetchone()
                    #next jump
                    if row == None or (row[3] != first[3]):
                        break
                ret.append(tag_typ)
                first = row
            return ret
        finally:
            cursor.close()
            connection.close()
            
    def addTag(self, tag_id, name, desc, rgb_color, tag_typ_id):
        """
        adds an tag to the database
        tag_typ_id the id of the tag_typ
        tag_id = NULL > autoincrement
        
        RETURNS: the autoincremented tag_id
        """
        #php xml-rpc correction
        if tag_id == "":
            tag_id = None
        connection = sqlite3.connect(self.db_file, isolation_level=None)
        cursor = connection.cursor()
        try:
            cursor.execute("BEGIN TRANSACTION")
            cursor.execute("INSERT INTO tag (tag_id,tag_name, tag_desc, color_rgb,tag_typ_id) VALUES (?,?,?,?,?);" , (tag_id, name, desc, rgb_color, tag_typ_id))
            newTag_id = cursor.lastrowid
            cursor.execute("COMMIT")
            return newTag_id
        except sqlite3.IntegrityError:
            raise SategaFault(CD_DB_NAME_NOT_UNIQUE, "The Name is not unique")
        except Exception:
            cursor.execute("ROLLBACK")
            raise
        finally:
            cursor.close()
            connection.close()
    
    def getTag(self, tag_id):
        """
        returns the data of the tag
        Array( tag_id , tag_name , tag_desc , tag_typ_id , tag_color)
        """
        connection = sqlite3.connect(self.db_file, isolation_level=None)
        cursor = connection.cursor()    
        try:
            data = []
            cursor.execute("SELECT tag.tag_id , tag.tag_name t, tag.tag_desc, tag.tag_typ_id , tag.color_rgb , tag_typ.color_rgb FROM tag join tag_typ on tag.tag_typ_id = tag_typ.tag_typ_id WHERE tag_id=?" , (int(tag_id),))
            row = cursor.fetchone()
            if row == None:
                return None
            data = row[0:4]
            finalRGB = ''
            if row[4] == "":
                finalRGB = row[5]
            else:
                finalRGB = row[4]
            data = data + (finalRGB,)
            return data
        except Exception:
            raise
        finally:
            cursor.close()    
            connection.close()
            
    def delTag(self, tag_id):    
        """
        when a tag is removed also the cd's that had this tag
        loose the tag.
        the cd is not deleted
        """
        connection = sqlite3.connect(self.db_file, isolation_level=None)
        cursor = connection.cursor()    
        try:
            cursor.execute("BEGIN TRANSACTION")
            cursor.execute("DELETE FROM tag WHERE tag_id=?" , (int(tag_id),))
            cursor.execute("COMMIT")
        except Exception:
            cursor.execute("ROLLBACK")
            raise
        finally:
            cursor.close()    
            connection.close()
                
    def delTagTyp(self, tag_typ_id):
        """
        removed and tagtyp from the database
        """
        connection = sqlite3.connect(self.db_file, isolation_level=None)
        cursor = connection.cursor()
        
        try:
            cursor.execute("BEGIN TRANSACTION")
            cursor.execute("DELETE FROM tag_typ WHERE tag_typ_id=?" , (int(tag_typ_id),))
            cursor.execute("COMMIT")
        except Exception:
            cursor.execute("ROLLBACK")
            raise
        finally:
            cursor.close()            
            connection.close()
    def getTagTypList(self):
        """
        gets a list of all known tagtypes
        array (
            array ( tag_typ_id , tag_typ_name  , tag_typ_color),
            array ( tag_typ_id , tag_typ_name  , tag_typ_color),
            ...
        )
        """
        connection = sqlite3.connect(self.db_file, isolation_level=None)
        cursor = connection.cursor()
        try:
            ret = []
            cursor.execute("SELECT tag_typ_id , name , color_rgb FROM tag_typ;")
            for  row in cursor:
                ret.append(row)
            return ret
        except Exception:
            raise
        finally:
            cursor.close()
            connection.close()        
        
    def addTagTyp(self, tag_typ_id, name, rgb_color):
        """
        adds a new tagtyp to the database
        parameter:
        tag_typ_id if NULL then autoincrement is used
        """
        #php xml-rpc correction
        if tag_typ_id == "":
            tag_typ_id = None
        connection = sqlite3.connect(self.db_file, isolation_level=None)
        cursor = connection.cursor()
        try:
            cursor.execute("BEGIN TRANSACTION")
            cursor.execute("INSERT INTO tag_typ (tag_typ_id,name,color_rgb) VALUES (?,?,?);" , (tag_typ_id, name, rgb_color))
            newTagTyp_id = cursor.lastrowid
            cursor.execute("COMMIT")
            return newTagTyp_id
        except sqlite3.IntegrityError:
            raise SategaFault(CD_DB_NAME_NOT_UNIQUE, "The Name is not unique")
        except Exception:
            cursor.execute("ROLLBACK")
            raise
        finally:
            cursor.close()
            connection.close()
                        
    def delCd(self, cd_id):
        """
        removed one cd from the database
        """
        connection = sqlite3.connect(self.db_file, isolation_level=None)
        cursor = connection.cursor()
        try:
            cursor.execute("BEGIN TRANSACTION")
            cursor.execute("DELETE FROM cd WHERE cd_id=?" , (long(cd_id),))
            cursor.execute("COMMIT")
        except Exception:
            cursor.execute("ROLLBACK")
            raise
        finally:
            cursor.close()
            connection.close()
    
    def updateCd(self, cd_id, cd_name, cd_desc, tag_ids):
        """
        updates the cd
        """
        if cd_id == "":
            cd_id = None
        
        # if php supplied an dict then only use the values
        if isinstance(tag_ids, dict):
            tag_ids = tag_ids.values()
        connection = sqlite3.connect(self.db_file, isolation_level=None)
        cursor = connection.cursor()
        try:
            cursor.execute("BEGIN TRANSACTION")
            cursor.execute("DELETE FROM cd WHERE cd_id=?" , (long(cd_id),))
            cursor.execute("INSERT INTO cd (cd_id,cd_name,cd_desc) VALUES (?,?,?);", (cd_id, cd_name, cd_desc))
            cd_id = cursor.lastrowid
            data = []
            for tag_id in tag_ids:
                data.append((cd_id, int(tag_id)))
            
            cursor.executemany("INSERT INTO cd_tag (cd_id,tag_id) VALUES (?,?);", data)

            cursor.execute("COMMIT")
            return cd_id
        except Exception:
            cursor.execute("ROLLBACK")
            raise
        finally:
            cursor.close()
            connection.close()
            
    def addCd(self, cd_id, cd_name, cd_desc, tag_ids):
        """
        adds an new cd to the database.
        tag_ids is an array with tag_id INT values.
        ldap_auth is checked in the agent.py file.
        If the optional parameter cd_id is 0
        the system will use an autoincrement number
        otherwise it will use the supplied id.
        
        RETURNS: the created cd_id
        """
        #php xml-rpc correction
        if cd_id == "":
            cd_id = None
        
        # if php supplied an dict then only use the values
        if isinstance(tag_ids, dict):
            tag_ids = tag_ids.values()
        connection = sqlite3.connect(self.db_file, isolation_level=None)
        cursor = connection.cursor()
        try:
            cursor.execute("BEGIN TRANSACTION")
            cursor.execute("INSERT INTO cd (cd_id,cd_name,cd_desc) VALUES (?,?,?);", (cd_id, cd_name, cd_desc))
            cd_id = cursor.lastrowid
            data = []
            for tag_id in tag_ids:
                data.append((cd_id, int(tag_id)))
            
            cursor.executemany("INSERT INTO cd_tag (cd_id,tag_id) VALUES (?,?);", data)

            cursor.execute("COMMIT")
            return cd_id
        except Exception:
            cursor.execute("ROLLBACK")
            raise
        finally:
            cursor.close()
            connection.close()
