# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
# Svein Hansen <sveinh@gmail.com>
# aatdark <aatdark@gmail.com>
# This file is part of Satega (www.satega.org)
#
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
#


from ConfigParser import SafeConfigParser
from service import Servicelist
import logging
import os.path


class SategaModule(object):
    """ Main interface and super class for all the Satega Module implementations.
    
    """
    
    def __init__(self):
        
        # Ensure this superclass is not instanciated directly
        if self.__class__ is SategaModule:
            raise SategaFault(support.errorcodes.VIRTUAL_CLASS,
                              "SategaModule is a virtual superclass, must be inherited")

        #register this module @the ServiceList
        Servicelist.register(self)

        
    def write_config_file(self):
        """ Physically write the ini file for this module to file storage

        """
        module_name = self.__class__.__name__.lower()
        registry = Registry()
        ini_file = os.path.join(registry.getValue("inifilespath"),
                                "modules",
                                module_name) + ".ini"
        fp = file(ini_file, "w")
        self.module_config.write(fp)


 
    def initialize(self):
        """ Method which should be overloaded by the sub classes
        
        This method is called when the agent process loads a module.       
        
        """
        raise SategaFault(support.errorcodes.VIRTUAL_FUNC_NOT_IMPL,
                          "Module must implement 'initialize()' function")
    
        
    def setModuleLoading(self,load):
        """
        with this function you can stop satega to even load the module.
        """
        self.module_config.set('main', 'enabled', load)
        self.write_config_file()
        return True

    def isLoad(self):
        """
        Query if satega will load this module
        
        @return: boolean
        """
        return self.enabled
    
    # ------ Functions that should be overloaded by the subclass
    
    def primary_module(self):
        """ Each module that is essential for the back-end
        must overload this function and return True

        """
        return False    
    
    def getName(self):
        """
        this function should return the name for the module
        """
        pass
    
    def disable(self):
        """
        in this function the module should handle
        the deactivation of itself
        """
        pass
    
    def enable(self):
        """
        the module should enable itself
        """
        pass
    
    def isEnabled(self):
        """
        the module should responde whenever it is enabled
        or not
        """
        pass
    
    # -------- End functions that the subclass should overload
