# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
#
# This file is part of Satega (www.satega.org).
#
# aatdark <aatdark@gmail.com>
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
# Autor of this file:
# -> aatdark
#



from satega.model.background import RemoveFilesTask
from satega.model.background import MoveFilesTask
from satega.model.background import TaskControl
from satega.lib.shellProcess import ShellProcess,shLaunch
import os
import tempfile
import shutil
import re
from ConfigParser import ConfigParser, NoOptionError, NoSectionError

import logging
log = logging.getLogger(__name__)

import formencode
class UniqueShare(formencode.FancyValidator):
	def _to_python(self, value, state):
            
            if value in ShareTools.get_samba_sections():
                    raise formencode.Invalid(
                            'The share already exists',
                            value, state)
            return value

class Share(object):
    def __init__(self):
        # if true then then the share already exists
        self.loaded = False
        self.writeable = False
        self.browseable = False
        self.sharename = None
        self.guest = False
        self.path = None
        self.comment = ''
        self._oldsharename = None
        self._old_datastore = None
        #the partition on which the share resists
        self.datastore = None
        self.nfs_enabled = False # nfs 
    def get_name(self):
        return self.sharename

    def get_path(self):
        return self.path

    def __repr__(self):
        return "<@Share %s %s>" % ( self.sharename , self.path)

    def __eq__(self,b):
        """
        makes an str cmp possible
        """
        if isinstance(b,basestring):
            if self.sharename == b:
                return True
        return False

    def save(self):
        #log.debug('Share save called %s' , self.loaded)
        if self.loaded:
            #update
            if self._old_datastore != self.datastore:
                #datastore has changed
                #start background worker to move files
                tc = TaskControl.get_instance()
                orig = '%s/shares/%s' % (self._old_datastore.getMountPoint(), self._oldsharename)
                to = '%s/shares/%s' % (self.datastore.getMountPoint(), self.sharename)
                task = MoveFilesTask('move files' , (orig , to))
                task.start()
                # @type tc TaskControl
                tc.register_task(task)
            NfsShareTools.remove_nfs_share(self._old_datastore.getMountPoint() + "/shares/" + self._oldsharename)
            mp = self.datastore.getMountPoint()
            path = mp + "/shares/"
            ShareTools.update_samba_share(self._oldsharename , self.sharename, self.comment, path, self.sharename, self.writeable, None, self.browseable)
            NfsShareTools.create_nfs_share(path + self.sharename, self.writeable)
        else:
            #create
            mp = self.datastore.getMountPoint()
            if mp.endswith('/'):
                path = mp + "shares/"
            else:
                path = mp + "/shares/"
            if self.nfs_enabled:
                NfsShareTools.create_nfs_share(path+self.sharename, self.writeable) #create nfs
            return ShareTools.create_samba_share(self.sharename, self.comment, path, self.sharename, self.writeable, None, self.browseable)
            
class ShareTools(object):

    @staticmethod
    def get_partition_from_path(path):
        """
        searches for the partition to the given path
        @path : like /mnt/devsdc1/myshare
        @returns: the partition object on which the path is
        """

        from satega.model.drive import DriveTools , Partition , Disk
        root = None
        #first check raid
        for part in DriveTools.getRaidDeviceObjects():
            mp =  part.getMountPoint()
            # @type path str
            if path.startswith(mp):
                return part
        for drive in DriveTools.getDriveList(only_ready_to_use = True):
            # @type drive Disk
            for part in drive.getPartitions(only_mounted = True):
                mp =  part.getMountPoint()
                if mp == '/':
                    #root filesystem
                    root = part
                    continue
                # @type path str
                if path.startswith(mp):
                    return part
        return root
    @staticmethod
    def get_share_object(shareName):
        """
        loads an section into a share object
        @returns the share object
        """


        smbConfig = ShareTools.read_samba_config()
        sharedetail  = ShareTools.get_share_detail(shareName, smbConfig)
        share = Share()
        share.writeable = sharedetail['writeable']
        share.browseable = sharedetail['browseable']
        share._oldsharename = share.sharename = sharedetail['sharename']
        share.guest = sharedetail['guest ok']
        share.path = sharedetail['path']
        share.comment = sharedetail['comment']
        share._old_datastore = share.datastore = ShareTools.get_partition_from_path(share.path)
        share.loaded = True # mark as already existing in the sabma config

        nfsshares = NfsShareTools.get_nfs_shares()
        if share.path in nfsshares:
            share.nfs_enabled = True
        else:
            share.nfs_enabled = False
        return share
    @staticmethod
    def get_all_share_objects():
        """
        returns all known shares as objects
        @returns all shares except printer , etc
        """
        smbConfig = ShareTools.read_samba_config()
        ret = []
        for section in ShareTools.get_samba_sections(smbConfig):
            obj = ShareTools.get_share_object(section)
            ret.append(obj)
        return ret

    @staticmethod
    def get_samba_sections(smbConfig = None):
        """
        Get all samba shares defined in the system

        @param smbConfig: Config parser holding smb.conf
        @return: an array [] of share names

        """
        if smbConfig == None:
            smbConfig = ShareTools.read_samba_config()
        sambaShares = []

        for section in smbConfig.sections():
            if not section in ["global", "printers", "print$"]:
                sambaShares.append(section)
        return sambaShares
    @staticmethod
    def get_samba_status(self):
        """
        Get the status of the samba system

        @return: Array of strings describing the status
        """
        cmd = shLaunch("smbstatus")
        outputString = str(cmd.stdout)
        returnStrings = []

        for line in outputString.splitlines():
            if line.startswith('WARNING') == False:
                returnStrings.append(line)

        return returnStrings

    @staticmethod
    def getSambaGlobals(self):
        """
        Retrieve all global keys and variables from the samba configuration

        @return: Dictionary of global values from the samba configuration

        Dictionary:
            workgroup: name of the samba workgroup
            server string: server string (server description)
        """
        # read the config file from disk
        smbConfig = self.__readSambaConfigFile()

        sambaGlobals = {}

        sambaGlobals["workgroup"] = smbConfig.get("global", "workgroup")
        sambaGlobals["server string"] = smbConfig.get("global", "server string")

        return sambaGlobals

    @staticmethod
    def get_share_detail(shareName, smbConfig):
        """
        Get the details of a samba share

        @param shareName: name of the share
        @param smbConfig: Config parser holding smb.conf
        @return: Dictionary of share details

        Dictionary:
            sharename: logical name of share
            path: path name to share folder
            writeable: True / False
            browseable: True / False
            comment: share comment
            guest ok: True / False
            valid users: string of valid users
        """

        sambaShareDetail = {}

        try:
            sambaShareDetail['sharename'] = shareName
            sambaShareDetail['path'] = smbConfig.get(shareName, 'path')

            if smbConfig.has_option(shareName, 'read only'):
                sambaShareDetail['writeable'] = True
            else:
                sambaShareDetail['writeable'] = False

            if smbConfig.has_option(shareName, 'browseable'):
                sambaShareDetail['browseable'] = False
            else:
                sambaShareDetail['browseable'] = True

            if smbConfig.has_option(shareName, 'comment'):
                sambaShareDetail['comment'] = smbConfig.get(shareName, 'comment')
            else:
                sambaShareDetail['comment'] = ''

            if smbConfig.has_option(shareName, 'guest ok'):
                sambaShareDetail['guest ok'] = True
            else:
                sambaShareDetail['guest ok'] = False

            if smbConfig.has_option(shareName, 'valid users'):
                sambaShareDetail['valid users'] = smbConfig.get(shareName, 'valid users')
            else:
                sambaShareDetail['valid users'] = ''
        except:
            raise Exception(-1, "Share does not exist: " + shareName)

        if len(sambaShareDetail) == 0:
            log.error("Share does not exist: " + shareName)
            raise Exception(-1, "Share does not exist: " + shareName)

        return sambaShareDetail

    @staticmethod
    def write_samba_config(smbConfig):
        """
        Write the Samba configuration file to disk

        @param smbConfig: Config parser holding smb.conf
        @return:
        """

        out = '\n'

        # Format the config parameters into .ini file format
        for section in smbConfig.sections():
            out = out + '[' + section + ']\n'
            for (key, value) in smbConfig.items(section):
                keyvalue = '\t' + key + ' = ' + value + '\n'

                out = out + keyvalue
            out = out + '\n\n\n'

        #print out
        f = None
        tmpfname = None
        try:
            # Write the new config file to a temporary file
            handle, tmpfname = tempfile.mkstemp("satega")
            f = os.fdopen(handle, "wb")
            f.write(out)
            f.flush()
            try:
                # Test if the new config file is correct
                shLaunch("testparm -v -s %s" % tmpfname)
            except IOError:
                raise Exception(-1,
                                  'Could not read smbconfig file: %s' % tmpfname)
            except Exception:
                raise Exception(-1, 'Invalid SMB config root')
            f.close()

            # The new config file is OK, move it to it's correct location and restart samba
            shutil.copyfile(tmpfname, '/etc/samba/smb.conf')
            log.info('New smb.conf written')
            ShareTools.restart_samba()
        except IOError, ioe:
            raise Exception(-1,
                              'Error writing new smbconfig file: %s' (ioe))
        finally:
            if f != None:
                f.close()
                os.remove(tmpfname)

    @staticmethod
    def restart_samba():
        """
        Restart the samba daemon (don't wait)
        
        """

        sh = ShellProcess("%s restart " % '/etc/init.d/samba')
        sh.execute_async()
        log.info("Samba daemon restart requested")

    @staticmethod
    def read_samba_config():
        """
        Read the Samba configuration file (/etc/samba/smb.conf)

        @return: ConfigParser

        """
        smbConfig = ConfigParser()
        try:

            cmd = shLaunch("testparm -v -s %s " % '/etc/samba/smb.conf')
            confString = str(cmd.stdout)

            # Strip whitespace and place into ConfigParser
            for line in confString.split("\n"):
                line = line.strip()
                s = re.search("^\[(.+?)\].*$", line)
                if s:
                    # Get section
                    section = s.group(1)
                    smbConfig.add_section(section)
                else:
                    # Get statement
                    stmt = re.search("^(.+?)=(.+)$", line)
                    if stmt:
                        option = stmt.group(1).strip()
                        value = stmt.group(2).strip()

#                        Ugly hack:
#                        This value ends with a '\' and messes up reading
#                        from the config-file as it is treated as an escape
#                        character by testparm samba validator
                        if option != "winbind separator":
                            smbConfig.set(section, option, value)
        except:
            raise Exception(-1,
                              "Error reading smb.conf file")

        if not smbConfig.sections():
            raise Exception(-2,
                              "Error reading smb.conf file")

        return smbConfig

    @staticmethod
    def is_share(shareName, smbConfig = None):
        """
        Private method to check if a share already exists

        @param shareName: name of the share
        @param smbConfig: Config parser holding smb.conf
        @return: True or False
        """
        if smbConfig == None:
            smbConfig = ShareTools.read_samba_config()
        for share in ShareTools.get_samba_sections(smbConfig):
            if share == shareName:
                return True

        return False

    @staticmethod
    def remove_samba_share(shareName):
        """
        Delete a Samba share

        @param shareName: name of the share to delete
        """
        # refresh the configuration file from disk
        smbConfig = ShareTools.read_samba_config()
        share = ShareTools.get_share_object(shareName)
        if not ShareTools.is_share(shareName, smbConfig):
            raise Exception(-1, "Share does not exist: %s" % shareName)

        if smbConfig.remove_section(shareName) == False:
            raise Exception(-1,
                              'Unable to remove share from smb config: %s' % shareName)

        ShareTools.write_samba_config(smbConfig)
        # @type share Share
        #remove the shares folder - asyn
        #shLaunch('rm -Rf %s/shares/%s' % ( share.datastore.getMountPoint() , shareName))
        tc = TaskControl.get_instance()
        path = '%s/shares/%s' % ( share.datastore.getMountPoint() , shareName)
        task = RemoveFilesTask('remove shares files %s' % (shareName), (path,))
        # @type tc TaskControl
        tc.register_task(task)
        task.start()
        log.info("Deleted Samba share: %s" % shareName)
        return True
    @staticmethod
    def update_samba_share(
                      oldshareName,
                      shareName,
                      comment,
                      path,
                      folder,
                      writeable,
                      validUsers,
                      browseable):
        """
        update an existing share
        """
        ShareTools.remove_samba_share(oldshareName)
        ShareTools.create_samba_share(shareName, comment, path, folder, writeable, validUsers, browseable)

  
    @staticmethod
    def create_samba_share(
                      shareName,
                      comment,
                      path,
                      folder,
                      writeable,
                      validUsers,
                      browseable):
        """
        Create a new Samba share

        @param shareName: logical name of the new share
        @param comment: share comment
        @param path: share base path
        @param folder: share file system folder name
        @param writeable: True or False
        @param validUsers: list of valid users
        @param browseable: True or False

        @return: Dictionary of share details. See __getSambaShareDetails()

        """
        # Validate input parameters
        if (shareName == ''):
            raise SategaFault(INVALID_PARAMETER,
                              "sharename cannot be empty")
        if (path == ''):
            raise SategaFault(INVALID_PARAMETER,
                              "path cannot be empty")
        if (folder == ''):
            raise SategaFault(INVALID_PARAMETER,
                              "folder cannot be empty")

        # refresh the configuration file from disk
        smbConfig = ShareTools.read_samba_config()

        if ShareTools.is_share(shareName, smbConfig):
            raise Exception(-1, "Cannot create share, it already exists: %s" % shareName)


        # Check that the path is existing in the file system
        if not os.path.isdir(path):
            os.mkdir(path)
            #raise Exception(-1, "Path does not exist: %s" % path)

        # if the share folder does not exist, create it
        sambaPath = os.path.join(path, folder)
        if not os.path.isdir(sambaPath):
            os.mkdir(sambaPath)
            os.chmod(sambaPath, 0777)

        # add the share information to smb.conf
        smbConfig.add_section(shareName)
        smbConfig.set(shareName, 'comment', comment)
        smbConfig.set(shareName, 'path', sambaPath)
        if writeable:
            smbConfig.set(shareName, 'writeable', 'yes')
            smbConfig.set(shareName, 'read only', 'no')
        else:
            smbConfig.set(shareName, 'writeable', 'no')
            smbConfig.set(shareName, 'read only', 'yes')
        if browseable:
            smbConfig.set(shareName, 'browseable', 'yes')
        else:
            smbConfig.set(shareName, 'browseable', 'no')

        if(validUsers == '' or validUsers == None):
            smbConfig.set(shareName, 'guest ok', 'yes')
            smbConfig.set(shareName, 'valid users', '')
        else:
            smbConfig.set(shareName, 'guest ok', 'no')
            smbConfig.set(shareName, 'valid users', validUsers)

        ShareTools.write_samba_config(smbConfig)
        return True

class NfsShareTools(object):
    """
    create/delete nfs shares
    """

    @staticmethod
    def remove_nfs_share(folder):
        """
        removes a nfs share from the server
        """
        print folder
        newlines = []
        exports = open('/etc/exports','r+')
        lines = exports.read().split(os.linesep)
        for line in lines:
            if not line.startswith(folder) and not line == "":
                newlines.append(line + os.linesep)
        print newlines
        exports.seek(0)
        exports.truncate()
        exports.writelines(newlines)
        exports.close()
        
    @staticmethod
    def get_nfs_shares():
        """
        @returns a list of folder that are shared with nfs
        """
        ret = []
        exports = open('/etc/exports','r')
        lines = exports.read().split(os.linesep)
        for line in lines:
            # @type line String
            if line.startswith('#'):
                continue
            parts = line.split(' ')
            ret.append(parts[0])
        exports.close()
        return ret

    @staticmethod
    def discovery_remote_nfs(hostname):
        """
        discovery the folder shared by another
        nfs server
        @param hostname the hostname to check
        @returns a list with folders
        sudo showmount -e 127.0.0.1
        """
        ret = {}
        sh = ShellProcess('showmount -e %s'%(hostname,))
        sh.execute()
        lines = sh.stdout.split(os.linesep)
        for line in lines:
            if not line.startswith('/'):
                continue
            parts = line.split(' ')
            ret.append(parts[0])
        return ret

    @staticmethod
    def reload_nfs_server():
        """
        reload the server with the (async)
        exportfs -ra
        cmd
        """
        sh = ShellProcess('exportfs -ra')
        sh.execute_async()
        log.info("NFS daemon restart requested")
        return True

    @staticmethod
    def create_nfs_share(folder,writeable):
        """
        creates a new nfs share in the /etc/exports file
        DOES NOT INVOKE nfs reload
        """
        if folder in NfsShareTools.get_nfs_shares():
            raise Exception('already a nfs share')
        exports = open('/etc/exports','a')
        if writeable == True:
            write = 'rw'
        else:
            write = 'ro'
        ip = '0.0.0.0/0.0.0.0'
        exports.write(os.linesep + '%s %s(%s,sync,no_subtree_check)'% (folder,ip,write) + os.linesep)
        exports.close()
