# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
#
# This file is part of Satega (www.satega.org).
#
# Aatdark <aatdark@gmail.com>
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
# Autor of this file:
# -> aatdark
#
import os
import ldap
import formencode
from satega.lib.shellProcess import ShellProcess,shLaunch
from pylons import config


import logging
log = logging.getLogger(__name__)

class LdapUser(object):
    #attribs = {}
    #isAdmin = False
    #password = None
    #dn = ""
    #dirty variables
    #pwDirty = False
    #adminDirty = False
    #flag for update or create
    #update = False
    
    def __init__(self,paramArray,isAdmin):
        self.attribs = paramArray
        self.dn = self.attribs['dn']
        self.isAdmin = isAdmin
        self.password = None
        self.pwDirty = False
        self.adminDirty = False
        self.update = False
        self.img = None
    
    def uid(self):
        return self.attribs['uid'][0]
    def firstname(self):
        if 'firstname' in self.attribs:
            return self.attribs['firstname']
        else:
            return ''
    def surname(self):
        if 'surname' in self.attribs:
            return self.attribs['surname']
        else:
            return ''
    
    def getDN(self):
        return self.dn
    
    def hasImg(self):
        if 'jpegPhoto' in self.attribs:
            return True
        else:
            return False
    
    def getImg(self):
        """
        returns the user image
        """
        return LdapControl.getUserImage(self.dn)
        
    def setImg(self,img):
        """
        sets the image of the current user
        """
        self.img = img
    def setPassword(self,pw):
        self.password = pw
        self.pwDirty = True
        
    def setAdmin(self,admin):
        """
        mark the admin to be set
        """
        if self.isAdmin != admin:
            self.adminDirty = True
            self.isAdmin = admin
    
    def save(self,ldapConnection):
        """
        saves the changes to the ldap server
        Parameter: needs an ldapConnection (this function will not close it afterwards)
        """
        if self.update:
            return self._update(ldapConnection)
        else:
            return self._create(ldapConnection)
    
    def _update(self,ldapConnection):
        from group import LdapGroup,LdapGroupControl
        
        """
        writes back the data for the current user
        """
        uid = self.uid()
        dn = self.dn
        
        modlist = []
        if self.img != None:
            LdapControl.setUserImage(dn,self.img)
        if self.pwDirty:
            newPassword  = self.password
            
            #update the password
            try:
                p = ShellProcess("smbldap-passwd %s" % uid)
                p.input = u"%s%s%s%s" % (newPassword, os.linesep, newPassword ,os.linesep)
                log.info('changing pw %s' % (uid))
                p.execute()
                if p.returncode != 0:
                    log.error('problem changeing passwords %s %s' % (p.stdout,p. stderr))
                    raise Exception('Can not Change user password %s %s' % (p.stdout,p. stderr))
            except Exception, e:
                raise Exception(0, str(e))
        if self.adminDirty:
            #update the admin
            admins = LdapGroupControl.loadGroup('admins')
            
            if self.isAdmin == False:
                admins.members.remove(self.dn)
            else:
                admins.members.append(self.dn)
            admins.save()
        return True
        
    def _create(self,ldapConnection):
        """
        creates a new user
        """
        admin = self.isAdmin
        uid = self.uid()
        dn = self.dn
        
        group_str = " "
        if admin == True :
            group_str = " -G 544"
        try:
            cmd = shLaunch("smbldap-useradd -a %s %s" % (group_str , uid))
        except Exception, e:
            raise Exception(0, str(e))
         
        #check again if it has been added
        if uid not in LdapControl.listUids():
            raise Exception("Undefined Problem occourd, the user has NOT BEEN ADDED")
        #add some ldap related stuff
        attributes = {
             "objectClass": ["inetOrgPerson", "organizationalPerson", "simpleSecurityObject", "sambaSamAccount", "posixAccount", "shadowAccount"],
        }
        #add all the users additional attributes
        
        attributes.update(self.attribs)
        modlist = []
        for key in attributes:
            #but the passwords needs to be handelnd with care
            #because we have to invoke the samba smbldap-passwd command
            if key == "dn":
                continue
            if key == "userPassword":
                self.__changeUserPassword(uid, attributes[key])
                continue
            val = attributes[key]
            if isinstance(val, unicode):
                val = val.encode()
            modlist.append((ldap.MOD_REPLACE , key , val))
        try:
            dn = LdapControl.makeDN(uid)
            ldapConnection.modify_s(dn, modlist)
        except Exception, error:
            raise
        
        #set the admin flag and the password
        user = LdapControl.loadUser(uid)
        user.setAdmin(self.isAdmin)
        user.setPassword(self.password)
        user.save(ldapConnection)
        if self.img != None:
            LdapControl.setUserImage(dn,self.img)
        
        return True
    
    def __str__(self):
        return "@LdapUser:"+self.uid()
        #return "@LdapUser:"+str(self.attribs)
class LdapControl(object):
    """
    the class manages to load user objects
    delete users and list users.
    
    in this class there are also the makeDN, makeUIDfromDN function
    to transforme the dn
    """
    
    @staticmethod
    def ping():
        """
        tests if the ldap server is reachable and accepts bind
        @return True if reachable and login succeeded
        """
        try:
            ldapConnection = LdapControl.getLdapConnection('root','satega')
        except Exception:
            return False
        return True
    
    @staticmethod
    def deleteUser(uid):
        """
        Deletes the user
        
        The funct removes the user from the admin group
        then executes an smbldap-userdel <username> command to delete the user.
        if the user was admin, he is removed from the admin group @ the ldap server
        @param uid the user login id like : root
        """
        if uid == 'root':
            raise Exception('you can not delete root')
        #remove the admin flag
        ldapConnection = LdapControl.getLdapConnection('root','satega')
        user = LdapControl.loadUser(uid)
        user.setAdmin(False)
        user.save(ldapConnection)
        
        dn = user.dn
        #remove from system
        try:
            cmd = shLaunch("smbldap-userdel %s" % uid)
        except Exception, e:
            raise Exception(TECHNICAL_SERVER_ERROR, str(e))
        
        return True
    
    @staticmethod
    def loadUser(uid):
        """
        returns the user object for the given uid
        returns none when the uid is not found
        """
        ldapConnection = LdapControl.getLdapConnection('root','satega')
        res = ldapConnection.search_s("ou=Users,dc=satega,dc=local", ldap.SCOPE_ONELEVEL, attrlist=("*",))
        
        for user in res:
            if "uid" in user[1] and user[1]['uid'][0] == uid:
                attribs = user[1]
                attribs['dn'] = user[0]
                isAdmin = LdapControl.checkIfAdmin(attribs['dn'])
                user = LdapUser(attribs,isAdmin)
                user.update = True
                ldapConnection.unbind_s()
                return user
            else:
                #nothing we ignore users without uid
                continue
        return None
    
    @staticmethod
    def listUids():
        """
        returns only the uid from the users known to the backend
        """
        ldapConnection = LdapControl.getLdapConnection('root','satega')
        res = ldapConnection.search_s("ou=Users,dc=satega,dc=local", ldap.SCOPE_ONELEVEL, attrlist=("uid",))
        users = []
        for user in res:
            #print user
            if "uid" in user[1]:
                users.append(user[1]["uid"][0])
            else:
                #nothing we ignore users without uid
                continue
        ldapConnection.unbind_s()
        return users
    
    @staticmethod
    def listUsers():
        """
        returns all the users known to the ldap backend
        """
        ldapConnection = LdapControl.getLdapConnection('root','satega')
        res = ldapConnection.search_s("ou=Users,dc=satega,dc=local", ldap.SCOPE_ONELEVEL, attrlist=("*",))
        users = []
        for user in res:
            #print user
            if "uid" in user[1]:
                #users.append(user[1]["uid"][0])
                attribs = user[1]
                attribs['dn'] = user[0]
                isAdmin = LdapControl.checkIfAdmin(attribs['dn'])
                user = LdapUser(attribs,isAdmin)
                user.update = True
                users.append(user)
            else:
                #nothing we ignore users without uid
                continue
        ldapConnection.unbind_s()
        return users
    
    @staticmethod
    def checkIfAdmin(dn):
        from group import LdapGroup,LdapGroupControl
        """
        checks if the user is admin or not.
        the parameter array is:
        1 username(dn)
        """
        if dn in LdapGroupControl.loadGroup('admins').members:
            return True
        else:
            return False
        try:      
            ldapConnection = LdapControl.getLdapConnection('root','satega')
            res = ldapConnection.search_s("cn=admins,ou=Groups,dc=satega,dc=local", ldap.SCOPE_BASE, attrlist=("uniqueMember",))
            ldapConnection.unbind_s()
            for user in res[0][1]["uniqueMember"]:
                if user == dn:
                    return True
            return False
        except ldap.LDAPError, error_message:
            #self.logger.debug("Couldn't Connect. %s " % error_message)
            ldapConnection.unbind_s()
            raise
    
    @staticmethod
    def getLdapConnection(uid,pw):
        """
        this method connects to the ldap server and returns the connection handler        
        """
        #ldap_server_ip = '192.168.143.128' # TODO move to config
        dn = LdapControl.makeDN(uid)
        ldapConnection = ldap.initialize(config['ldap_url'])
        ldapConnection.protocol_version = ldap.VERSION3    
        ldapConnection.simple_bind_s(dn, pw)
        return ldapConnection
    
    @staticmethod
    def setUserImage(dn,img):
        """
        sets the img for the user
        """
        try:
            ldapConnection = LdapControl.getLdapConnection('root','satega')
            res = ldapConnection.search_s(dn, ldap.SCOPE_BASE, attrlist=("*",))
            if img == '': #delete the img
                modlist = ((ldap.MOD_DELETE , 'jpegPhoto', None ),)
            else:
                modlist = ((ldap.MOD_REPLACE , 'jpegPhoto' , (img,)),)
            ldapConnection.modify_s(dn, modlist)
            return True
        except ldap.LDAPError, error_message:
            #self.logger.error("LDAP Connection Error: " + str(error_message))
            raise Exception("LDAP Connection Error: " + str(error_message))
            return False    
        except Exception , error:
            raise
        finally:
            ldapConnection.unbind_s()
    @staticmethod
    def getUserImage(dn):
        """
        gets only the hex image for an user
        parameter array is:
        1 username for the user the info is requested (dn syntax)
        """
        try:
            ldapConnection = LdapControl.getLdapConnection('root','satega')
            res = ldapConnection.search_s(dn, ldap.SCOPE_BASE, attrlist=("*",))

            for key in res[0][1]:
                if(key == "jpegPhoto"):
                    #return base64.b64encode(res[0][1][key][0])
                    return res[0][1][key][0]
                else:
                    continue
            return None
        except ldap.LDAPError, error_message:
            #self.logger.error("LDAP Connection Error: " + str(error_message))
            raise Exception("LDAP Connection Error: " + str(error_message))
            return False    
        except Exception , error:
            raise
        finally:
            ldapConnection.unbind_s()
    
    @staticmethod
    def UIDfromDN(dn):
        """
        opposit to makeDN
        """
        import re
        c = re.compile(r"uid=(\w+),ou=Users")
        m = c.match(dn)
        try:
            return m.group(1)
        except AttributeError:
            raise ValueError('not in dn syntax')
            
    @staticmethod
    def makeDN(uid):
         """
         makes the simple username to a DN (needes for ldap auth)
         @param uid : the user login id like : root
         @return the dn like uid=root,ou=Users,dc=satega,dc=local
         """
         return "uid=" + uid + ",ou=Users,dc=satega,dc=local"
        
    
class UniqueUsername(formencode.FancyValidator):
	def _to_python(self, value, state):
		if value in LdapControl.listUids():
			raise formencode.Invalid(
				'That username already exists',
				value, state)
		return value
