# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
#
# This file is part of Satega (www.satega.org). 
#
# aatdark <aatdark@gmail.com>
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
# Autor of this file:
# -> aatdark
#

if __name__ == "__main__":
    print "x"*60
    print "IMPORTANT:: Please run with nosetests --with-pylons development.ini"
    print "x"*60
import unittest
from satega.model.drive import DriveTools

class TestDriveTools(unittest.TestCase):
    def testgetAttribsofMountedDevices(self):
        #test wrong usage ( device instead of partition)
        ret = DriveTools.getAttribsofMountedDevices('/dev/sda')
        assert ret == False
        ret = DriveTools.getAttribsofMountedDevices('/dev/sda1')
        print ret
        assert ret != False
    def testmountDevice(self):
        #try to unmount to make static state
        mt = DriveTools.getMountPointForPartition('/dev/sdb1')
        if mt != None:
            DriveTools.unmountDevice(mt)
        mountpoint = DriveTools.mountDevice('/dev/sdb1','')
        print mountpoint
        assert mountpoint != ''
        assert len(mountpoint ) > len('/dev/')
        ret = DriveTools.getAttribsofMountedDevices('/dev/sdb1')
        assert ret['mountpoint'] == mountpoint
        mt2 = DriveTools.getMountPointForPartition('/dev/sdb1')
        assert mountpoint == mt2
        
        DriveTools.unmountDevice(mountpoint)
        ret = DriveTools.getAttribsofMountedDevices('/dev/sdb1')
        print ret
        assert ret == False
        mt2 = DriveTools.getMountPointForPartition('/dev/sdb1')
        assert mt2 == None
    
    def testgetDeviceCapacity(self):
        cap = DriveTools.getDeviceCapacity('/dev/sda')
        print cap
        assert cap[0] > 10
        cap1 = DriveTools.getDeviceCapacity('/dev/sda1')
        assert cap[0] > cap1[0]
        assert cap[1] == 'MB'
        assert cap1[1] == 'MB'
        #test device that is not existing
        cap = DriveTools.getDeviceCapacity('/dev/xxx123123')
        assert len(cap) == 0
        
    def testcheckIfDevIsPartOfRaid(self):
        #needs some work
        assert( DriveTools.checkIfDevIsPartOfRaid('/dev/sda') == False)
        
    def testgetPartitionsForDevice(self):
        #needs some review
        parts = DriveTools.getPartitionsForDevice('/dev/sdb')
        print parts
        assert len(parts) > 0
        assert parts[0] != None
        assert parts[0]['devname'] == '/dev/sdb1'
        assert parts[0]['filesystem'] == 'ext3'
        assert parts[0]['size'][0] > 0
        if parts[0]['mountpoint'] == None:
            assert parts[0]['free'] == 0
            assert parts[0]['used'] == 0
        else:
            assert parts[0]['free'] > 0
            assert parts[0]['used'] > 0
    
    def testgetFreeUsedDiskSpace(self):
        ret  = DriveTools.getFreeUsedDiskSpace('/dev/sdb')
        print ret
        assert len(ret) == 2
        assert ret['used'] > 0
        assert ret['free'] > 0
        
    def testgetDeviceFiles(self):
        devs = DriveTools.getDeviceFiles()
        print devs
        assert len(devs) > 0
        # check if realy a dev string
        assert len(devs[0]) > 0
        assert devs[0].startswith('/dev/') == True
        assert '/dev/sda' in devs
        #no partitions
        assert '/dev/sda1' not in devs
        assert '/dev/sdb' in devs
    def testgetDriveList(self):
        drives = DriveTools.getDriveList()
        print drives
        assert len(drives)>=2
        assert '/dev/sda' in drives
        assert drives[0].getDevPath() == '/dev/sda'
        assert drives[0].getSize() > 2
        partitions = drives[0].getPartitions()
        assert len(partitions) >= 1
        assert partitions[0].devname.startswith(drives[0].getDevPath()) == True
        assert partitions[0].mountpoint != None
        assert partitions[0].filesystem == 'ext3'
    def testgetPartition(self):
        part = DriveTools.getPartition('/dev/sda1')
        assert part != False
        assert part.getFilesystem() != ''
        assert part.getMountPoint() == '/'
    def testgetParameterofRaidDev(self):
        devname = '/dev/md0'
        res = DriveTools.getParameterofRaidDev(devname)
        print res
        assert res['size'] > 0
        assert res['raidlevel'] >= 0
        
    def testgetRaidDeviceObjects(self):
        parts = DriveTools.getRaidDeviceObjects()
        print parts
        assert len(parts) > 0
        assert parts[0].getDevPath().startswith('/dev/md')
        print parts[0].getSize()
        assert parts[0].getSize() > 0
        if parts[0].getMountPoint() != None:
            assert parts[0].getFree() > 0
        else:
            assert parts[0].getFree() == 0
    def testautoMountAllParts(self):
        #unmount all parts
        drives = DriveTools.getDriveList()
        for drive in drives:
            for part in drive.getPartitions():
                if part.getFilesystem() == 'linux-swap':
                    continue
                part.unmount()
                assert part.isMounted() == False
        
        #now the real test
        DriveTools.autoMountAllParts()
        drives = DriveTools.getDriveList()
        for drive in drives:
            for part in drive.getPartitions():
                if part.getFilesystem() == 'linux-swap':
                    continue
                assert part.isMounted() == True
        #and unmount again
        drives = DriveTools.getDriveList()
        for drive in drives:
            for part in drive.getPartitions():
                if part.getFilesystem() == 'linux-swap':
                    continue
                part.unmount()
                assert part.isMounted() == False
