# -*- coding: utf-8; -*-
#
# Copyright (C) 2010
#
# This file is part of Satega (www.satega.org).
#
# aatdark <aatdark@gmail.com>
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
# Autor of this file:
# -> aatdark
#

if __name__ == "__main__":
	print "x"*60
	print "IMPORTANT:: Please run with nosetests --with-pylons development.ini"
	print "x"*60
import unittest
from satega.model.user import LdapControl,LdapUser

class TestUser(unittest.TestCase):
	def testCreate(self):
		if 'frankxx' in LdapControl.listUids():
			LdapControl.deleteUser('frankxx')
			if 'frankxx' in LdapControl.listUids():			
				raise Exception("user exists")
		params = { 
			"dn" : LdapControl.makeDN('frankxx') ,
			"uid" : ('frankxx',) ,
			}
		user = LdapUser(params , True)
		user.setPassword('fritz')
		
		assert user.uid() == 'frankxx'
		assert user.dn == 'uid=frankxx,ou=Users,dc=satega,dc=local'
		
		ldapConnection = LdapControl.getLdapConnection('root','satega')
		user.save(ldapConnection)
		
		userL = LdapControl.loadUser('frankxx')
		assert userL.uid() == 'frankxx'
		assert userL.dn == 'uid=frankxx,ou=Users,dc=satega,dc=local'
		assert userL.isAdmin == True
		
		userL.setAdmin(False)
		assert userL.isAdmin == False
		userL.save(ldapConnection)
		
		userLL = LdapControl.loadUser('frankxx')
		assert userLL.isAdmin == False

		assert  'frankxx' in LdapControl.listUids() 		
		LdapControl.deleteUser('frankxx')
		assert  'frankxx' not in LdapControl.listUids() 
		
class TestLdapControl(unittest.TestCase):
	def setUp(self):
		pass
		
	def testPing(self):
		assert LdapControl.ping()

	def testloadUser(self):
		uid = 'root'
		user = LdapControl.loadUser(uid)
		assert user != None
		assert user.getDN() == 'uid=root,ou=Users,dc=satega,dc=local'
		assert user.uid() == 'root'
		assert user.isAdmin  == True
	
	def testlistUids(self):
		uids = LdapControl.listUids()
		assert 'root' in uids
		
	def testlistUsers(self):
		users = LdapControl.listUsers()
		found = False
		for user in users:
			if user.uid() == 'root':
				found = True
				break
		assert found
		
	def testUIDfromDN(self):
		dn = 'uid=root,ou=Users,dc=satega,dc=local'
		assert LdapControl.UIDfromDN(dn) == 'root'
		
		dn = 'balbal'
		self.failUnlessRaises(ValueError,LdapControl.UIDfromDN,dn)
		
	def testmakeDN(self):
		uid = 'root'
		assert LdapControl.makeDN(uid) == 'uid=root,ou=Users,dc=satega,dc=local'
	
	def testcheckIfAdmin(self):
		assert LdapControl.checkIfAdmin(LdapControl.makeDN('root'))
		
	def testsetUserImage(self):
		res = LdapControl.setUserImage(LdapControl.makeDN('root'),'xxx')
		assert res

