#!/bin/bash
# check for admin rights; needed to run this script
if [ $UID != 0 ]
then
echo Please use sudo satega-install.sh
exit
fi

timestamp=`date '+%Y%m%d_%H%M'`

/etc/init.d/samba stop
/etc/init.d/slapd stop
#/etc/init.d/apache2 stop

echo 1. removing installed packages
apt-get purge -y python-twisted \
   python-ldap python-sqlite sqlite3 samba samba-common samba-doc \
   ldap-utils smbldap-tools python-dev mt-daapd inadyn libnss-ldap nfs-kernel-server
apt-get -y autoremove


echo 2. installing required packages
apt-get install -y  python-twisted parted\
   python-ldap python-sqlite sqlite3 samba samba-common samba-doc \
   ldap-utils smbldap-tools python-dev mt-daapd inadyn libnss-ldap nfs-kernel-server


echo 3. removing old ldap installation
apt-get purge -y slapd
rm -rf /etc/ldap
rm -rf /var/run/slapd
rm -rf /var/lib/ldap/

echo 4. install slapd
apt-get install -y slapd

echo 5. adding samba schema
zcat /usr/share/doc/samba-doc/examples/LDAP/samba.schema.gz > /etc/ldap/schema/samba.schema

echo 6. configuring slapd.conf...
mv /etc/ldap/slapd.conf /etc/ldap/slapd.conf.$timestamp
cp slapd.conf.satega /etc/ldap/slapd.conf


echo 7. configuring ldap.conf...
cp ldap.conf.satega /etc/ldap/ldap.conf

echo 8. installing smbldaptools
apt-get install -y smbldap-tools


#echo 9. configuring apache php
#mv /etc/php5/apache2/php.ini /etc/php5/apache2/php.ini.$timestamp
#cp php.ini.satega /etc/php5/apache2/php.ini
mv /etc/samba/smb.conf /etc/samba/smb.conf.$timestamp
cp smb.conf.satega /etc/samba/smb.conf

#echo 10. installing phpldapadmin
#apt-get install -y phpldapadmin

echo 11. Adding satega.ldif to ldap server
/etc/init.d/slapd restart
sleep 5
ldapadd -x -w satega -D 'cn=admin,dc=satega,dc=local' -f satega.ldif -c

echo 12. Configuring smbldap tools
cp smbldap.conf.satega.orig smbldap.conf.satega
sid=$(net getlocalsid | awk '{print $NF}')
sed -i s/SATEGASID/$sid/g smbldap.conf.satega
cp smbldap.conf.satega /etc/smbldap-tools/smbldap.conf
cp smbldap_bind.conf.satega /etc/smbldap-tools/smbldap_bind.conf

chmod 0644 /etc/smbldap-tools/smbldap.conf
chmod 0600 /etc/smbldap-tools/smbldap_bind.conf

smbldap-populate
smbpasswd -w satega


echo 13. Setting up PAM and NS
cp common-account.satega /etc/pam.d/common-account
cp common-auth.satega /etc/pam.d/common-auth
cp common-password.satega /etc/pam.d/common-password
cp common-session.satega /etc/pam.d/common-session
cp nsswitch.conf.satega /etc/nsswitch.conf

echo 14. Restarting daemons
#/etc/init.d/samba restart
/etc/init.d/slapd restart
#/etc/init.d/apache2 restart

echo 15. Additions

echo 15.1 Inadyn startup script and disable autostart
cp ../inadyn/inadyn /etc/init.d/inadyn
chmod 755 /etc/init.d/inadyn
/etc/init.d/inadyn stop
update-rc.d -f inadyn remove
   
echo 15.2 mt-daapd disable autostart
/etc/init.d/mt-daapd stop
update-rc.d -f mt-daapd remove

echo 15.3 Install miniupnpc python addon
here=`pwd`
sudo mkdir -p /usr/local/src
cd /usr/local/src
sudo tar xvfz $here/../miniupnpc/miniupnpc.tar.gz
cd miniupnpc
sudo make
sudo make installpythonmodule
cd $here





