# -*- coding: utf-8; -*-
#
# Copyright (C) 2009
# aatdark <aatdark@gmail.com>
#
# This file is part of Satega. (satega.org)
#
# Satega is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Satega is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Satega.  If not, see <http://www.gnu.org/licenses/>.
# Autor of this file:
# -> aatdark
#
from repoze.what.adapters import BaseSourceAdapter
import ldap
import ldap.modlist

class LDAPGroupSourceAdapter(BaseSourceAdapter):
	"""Mock group source adapter"""

	def __init__(self,ldapConnection,basedn,groupdn):
		"""
		ldapConnection: ldap String like ldap://127.0.0.1:389
		basedn: like ou=Users,dc=satega,dc=local
		groupdn : like ou=Groups,dc=satega,dc=local
		"""
		
		super(LDAPGroupSourceAdapter, self).__init__(writable=False)
		self.ldapConnection = ldapConnection
		self.basedn = basedn
		self.groupdn = groupdn
		
	def _make_conn(self):
		"""
		this function creates the ldapConnection object for all the methods
		"""
		ldapConnection = ldap.initialize(self.ldapConnection)
		#if you want to force v3
		#ldapConnection.protocol_version = ldap.VERSION3
		
		#if you have to authenticate against the ldap server
		#ldapConnection.simple_bind_s("uid=root,ou=Users,dc=satega,dc=local", 'satega')
		
		return ldapConnection

	def _get_all_sections(self):
		ldapConnection = self._make_conn()
		res = ldapConnection.search_s(self.groupdn, ldap.SCOPE_ONELEVEL,filterstr ='(objectClass=groupOfUniqueNames)', attrlist=("cn",'uniqueMember'))
		ret = {}
		for row in res:
			unique =  row[1]['uniqueMember']
			cn =  row[1]['cn'][0]
			ret[cn] = set(unique)
		ldapConnection.unbind_s()
		return ret

	def _get_section_items(self, section):
		ldapConnection = self._make_conn()
		res = ldapConnection.search_s(self.groupdn, ldap.SCOPE_ONELEVEL,filterstr ='(&(objectClass=groupOfUniqueNames)(cn='+section+'))', attrlist=("cn",'uniqueMember'))
		unique =  res[0][1]['uniqueMember']
		ldapConnection.unbind_s()
		return unique
		
	def _find_sections(self, credentials):

		username = credentials['repoze.what.userid']
		ldapConnection = self._make_conn()
		res = ldapConnection.search_s(self.groupdn, ldap.SCOPE_ONELEVEL,filterstr ='(&(objectClass=groupOfUniqueNames)(uniqueMember='+username+'))', attrlist=("cn",'uniqueMember'))
		ret = {}
		for row in res:
			unique =  row[1]['uniqueMember']
			cn =  row[1]['cn'][0]
			ret[cn] = set(unique)
		ldapConnection.unbind_s()
		return ret

